#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


import time 
import os
import matplotlib.pyplot as plt
from scipy.signal import savgol_filter
from scipy.interpolate import interp1d

import numpy as np
import pandas as pd

class Driver():
    
    slot_config = '<MODULE_NAME>,<CALIBRATION_PATH>'
    
    def __init__(self,**kwargs):
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name, calib_path = [a.strip() for a in kwargs[key].split(',')]
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,slot_num,name,calib_path))
                self.slot_names[slot_num] = name
        
        
    def get_driver_model(self):
        
        model = []
        for name in self.slot_names.values() :
            model.append({'element':'module','name':name,'object':getattr(self,name)})
        return model
        
        
    
#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        import visa
        
        
        self.BAUDRATE = 115200
        
        # Initialisation
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.baud_rate = self.BAUDRATE
        
        Driver.__init__(self,**kwargs)
        
        
        
    def close(self):
        try : self.controller.close()
        except : pass

    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\r\n')
        return result
    
    def write(self,command):
        self.controller.write(command)
        
        
############################## Connections classes ##############################
#################################################################################

class Module_NSR1():
    
    def __init__(self,dev,slot,name,calibPath):
        
        self.dev = dev
        self.NAME = name
        self.SLOT = slot
        
        self.check_notref_state()
    
        self.calibPath = calibPath
        assert os.path.exists(calibPath)
        self.calibration_function = None
        
        self.calib = None
        self.load_calibration()
        
        
        
    # Query write functions
    # =========================================================================  

    def query(self,command,unwrap=True) :
        result = self.dev.query(self.SLOT+command)
        if unwrap is True :
            try:
                prefix=self.SLOT+command[0:2]
                result = result.replace(prefix,'')
                result = result.strip()
                result = float(result)
            except:
                pass
        return result
        
    def write(self,command) :
        self.dev.write(self.SLOT+command)
    
       
    # Drivers functions
    # =========================================================================
            
    def get_id(self):
        return self.query('ID?')
    
    
    
    def get_filter_state(self):
        ans = self.query('TS?',unwrap=False)[-2:]
        if ans[0] == '0' :
            return 'REF'
        elif ans == '14' :
            return 'CONF'
        elif ans == '1E' :
            return 'HOMING'
        elif ans == '28' :
            return 'MOVING'
        elif ans[0] == '3' and ( ans[1].isalpha() is False)  :
            return 'ENABLED'
        elif ans[0] == '3' and ( ans[1].isalpha() is True)  :
            return 'DISABLED'
        else :
            return 'UNKNOWN'


        
    def check_notref_state(self):
        
        # On vérifie que l'on est pas dans le mode REF
        state=self.get_filter_state()
        if state == 'REF' :
            self.write('OR') # Perfom Home search
            while self.get_filter_state() == 'HOMING' :
                time.sleep(0.5)
        elif state == 'CONF' :    
            self.write('PW0') # Sortie du mode Configuration
            self.check_notref_state()
            
    def wait_move_ending(self):
        while self.get_filter_state() == 'MOVING' :
            time.sleep(0.1)

    
    
    
    def set_enabled(self,value):
        assert isinstance(bool(value),bool)
        self.write('MM'+str(int(value)))
    
    def is_enabled(self):
        state=self.get_filter_state()
        if state in ['ENABLED','MOVING'] :
            return True
        else :
            return False
        
        

        
        
    #--------------------------------------------------------------------------
    # Instrument variables
    #--------------------------------------------------------------------------
    
    
 
    
    def set_velocity(self,value):
        
        assert isinstance(int(value),int)
        value=int(value)
        
        if value != self.get_velocity():
            
            # Go to not ref mode
            self.write('RS')
            while self.get_filter_state() != 'REF':
                time.sleep(0.1)
            
            # Go to config mode
            self.write('PW1')
            while self.get_filter_state() != 'CONF':
                time.sleep(0.1)
            
            # Change value
            self.write('VA%i'%value)

            # Sortie du mode config
            self.write('PW0')
            while self.get_filter_state() != 'REF':
                time.sleep(0.1)
            
            # Homing to get back to ready state
            self.write('OR')
            while self.get_filter_state() != 'ENABLED':
                time.sleep(0.1)

            # On le désactive
            self.set_enabled(False)
            while self.get_filter_state() != 'DISABLED':
                time.sleep(0.1)
    
    def get_velocity(self):
        return self.query('VA?')
    
    
    def set_acceleration(self,value):
        
        assert isinstance(int(value),int)
        value=int(value)
        
        if value != self.get_velocity():
            
            # Go to not ref mode
            self.write('RS')
            while self.get_filter_state() != 'REF':
                time.sleep(0.1)
            
            # Go to config mode
            self.write('PW1')
            while self.get_filter_state() != 'CONF':
                time.sleep(0.1)
            
            # Change value
            self.write('AC%i'%value)

            # Sortie du mode config
            self.write('PW0')
            while self.get_filter_state() != 'REF':
                time.sleep(0.1)
            
            # Homing to get back to ready state
            self.write('OR')
            while self.get_filter_state() != 'ENABLED':
                time.sleep(0.1)

            # On le désactive
            self.set_enabled(False)
            while self.get_filter_state() != 'DISABLED':
                time.sleep(0.1)
    
        
    def get_acceleration(self):
        return self.query('AC?')
    
            
            
    
    
    
    def set_angle(self,value,forced=False):
        assert isinstance(float(value),float)
        value=float(value)
        
        if forced is False :
            curr_angle = self.get_angle()
            if value > curr_angle - 1.9 :
                self.set_angle(value+20,forced=True)
            
        self.set_enabled(True)
        self.write('PA'+str(value))
        self.wait_move_ending()
        self.set_enabled(False)
        
    def get_angle(self):
        value = self.query('TP?')
        return value
       
    
    
    
    
    
    def set_min(self):
        self.set_transmission(0)
        
    def set_max(self):
        self.set_transmission(1)





    def set_transmission(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        angle = self.get_angle_from_transmission(value)
        self.set_angle(angle)
    
    def get_transmission(self):
        angle = self.get_angle()
        return self.get_transmission_from_angle(angle) 

    def get_transmission_max(self):
        return self.calib.transmission.max()



    def get_angle_from_transmission(self,value):
        ind = abs(self.calib.transmission-value).idxmin()
        return float(self.calib.loc[ind,'angle'])   
    
    def get_transmission_from_angle(self,value):
        ind = abs(self.calib.angle-value).idxmin()
        return float(self.calib.loc[ind,'transmission'])   


    
    def go_home(self):
                            
        # Go to not ref mode
        self.write('RS')
        while self.get_filter_state() != 'REF':
            time.sleep(0.1)
        
        # Homing to get back to ready state
        self.write('OR')
        while self.get_filter_state() != 'ENABLED':
            time.sleep(0.1)

        # On le désactive
        self.set_enabled(False)
        while self.get_filter_state() != 'DISABLED':
            time.sleep(0.1)
            
            
            
    # Calibration functions
    # =========================================================================
    
    def load_calibration(self):
        try: self.calib = pd.read_csv(os.path.join(self.calibPath,'calib.csv'))
        except: pass
    
    def set_calibration_function(self,calibration_function):
        self.calibration_function = calibration_function
        
    def calibrate(self):
        
        assert self.calibration_function is not None
        
        def scan(list_angle):
            
            df = pd.DataFrame()

            for angle_setpoint in list_angle :
                self.set_angle(angle_setpoint)
                angle=self.get_angle()
                power=self.calibration_function()
                df=df.append({'angle':angle,'power':power})
                   
            df.sort_values(by=['angle'],inplace=True)
            return df
        
        plt.ioff()
        fig = plt.figure()
        ax=fig.add_subplot(111)
        
        # Homing
        self.go_home()
        
        # Fabrication de la liste des angles à explorer
        list_angle = np.concatenate((np.arange(0,360,8),np.arange(4,360,8)))
        list_angle = np.flipud(list_angle)
        
        # Lancement du scan de mesure de puissance
        df=scan(list_angle)
        
        # Find transition
        derivative = np.diff(df.power)
        imax=np.argmax(derivative)
        
        # Recoupage
        df_shifted = pd.DataFrame()
        df_shifted['power'] = np.concatenate((df.power[imax:],df.power[0:imax]))
        df_shifted['angle'] = np.concatenate((df.angle[imax:]-360,df.angle[0:imax]))
        df = df_shifted
        
        # Suppression transition sur 10deg
        df = pd.DataFrame()
        df = df[df.angle>(df.angle.min()+10)]
        df = df[df.angle<(df.angle.max()-10)]
        
        # Normalisation
        df['transmission']=df.power/df.power.max()
        ax.plot(df.angle,df.transmission,'x',label='raw')
        
        # Lissage
        df['transmission']=savgol_filter(df.transmission, 7, 3)
        
        # Interpolation
        interpolFunc=interp1d(df.angle, df.transmission, kind='cubic')
        df['angle']=np.linspace(df.angle.min(),df.angle.max(),1000)
        df['transmission']=interpolFunc(df.angle)
        
        # EXTRACT On ne garde que entre 0% et 95% car trop bruité au dessus
        df = df[df.transmission<0.95]
        
        # Sauvegarde plot
        date=time.strftime("%Y%m%d_%H%M%S")
        ax.plot(df.angle,df.transmission,'r-',label='Final')
        ax.set_title(f'{self.NAME} filter {date}')
        ax.grid()
        ax.set_xlabel('Angle [deg]')
        ax.set_ylabel('Power [a.u.]')
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(handles, labels, loc=0)
        filepath=os.path.join(self.calibPath,f'{date}_{self.NAME}_calib.jpg')
        fig.savefig(filepath,bbox_inches='tight',dpi=300)
        fig.close()
           
        # Enregistrement des données 
        df.to_csv(os.path.join(self.calibPath,'calib.csv'))
        
        # Raffraichissement des donnees
        self.load_calibration()
     
        
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'velocity','type':float,'read':self.get_velocity,'write':self.set_velocity,'help':'Velocity of the filter during move'})
        model.append({'element':'variable','name':'acceleration','type':float,'read':self.get_acceleration,'write':self.set_acceleration,'help':'Acceleration of the filter during move'})
        model.append({'element':'variable','name':'angle','type':float,'read':self.get_angle,'write':self.set_angle,'help':'Current angle position'})
        model.append({'element':'variable','name':'transmission','type':float,'read':self.get_transmission,'write':self.set_transmission,'help':'Current transmission of the filter'})
        model.append({'element':'action','name':'set_min','do':self.set_min,'help':'Go to minimum transmission'})
        model.append({'element':'action','name':'set_max','do':self.set_max,'help':'Go to maximum transmission'})
        model.append({'element':'action','name':'go_home','do':self.go_home,'help':'Go to home position'})
        return model
    
    
        
        
    



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:16:09 2019

@author: qchat
"""
from PyQt5 import QtWidgets
import autolab
from autolab import paths
import configparser
import datetime
import os
        
class ConfigManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        
        # Configuration menu
        configMenu = self.gui.menuBar.addMenu('&Configuration')
        self.importAction = configMenu.addAction('Import configuration')
        self.importAction.triggered.connect(self.importActionClicked)
        exportAction = configMenu.addAction('Export current configuration')
        exportAction.triggered.connect(self.exportActionClicked)

        # Initializing configuration values
        self.config = {}
        self.config['parameter'] = {'element':None,'name':None}
        self.config['nbpts'] = 11
        self.config['range'] = (0,10)
        self.config['log'] = False
        self.config['recipe'] = []
        
        
       
    # NAMES
    ###########################################################################
        
    def getNames(self,option=None):
        
        """ This function returns a list of the names of the recipe step and of the parameter """
        
        names = [step['name'] for step in self.config['recipe']]
        if self.config['parameter']['element'] is not None and option != 'recipe': 
            names.append(self.config['parameter']['name'])
        return names
        
    
    
    def getUniqueName(self,basename):
        
        """ This function adds a number next to basename in case this basename is already taken """
        
        names = self.getNames()
        name = basename
        
        compt = 0
        while True :
            if name in names :
                compt += 1
                name = basename+'_'+str(compt)
            else :
                break
        return name
    
    
        
    # CONFIG MODIFICATIONS
    ###########################################################################   
    
    def setParameter(self,element,name=None):
        
        """ This function set the element provided as the new parameter of the scan """
        
        if self.gui.scanManager.isStarted() is False :
            self.config['parameter']['element'] = element
            if name is None : name = self.getUniqueName(element.name)
            self.config['parameter']['name'] = name
            self.gui.parameterManager.refresh()
            self.gui.dataManager.clear()
            
            
            
    def setParameterName(self,name):
        
        """ This function set the name of the current parameter of the scan """
        
        if self.gui.scanManager.isStarted() is False :
            if name != self.config['parameter']['name']:
                name = self.getUniqueName(name)
                self.config['parameter']['name'] = name
                self.gui.dataManager.clear()
        self.gui.parameterManager.refresh()
        
        
        
    def addRecipeStep(self,stepType,element,name=None,value=None):
        
        """ This function add a step to the scan recipe """
        
        if self.gui.scanManager.isStarted() is False :
            
            if name is None : name = self.getUniqueName(element.name)
            step = {'stepType':stepType,'element':element,'name':name,'value':None}
            
            # Value
            if stepType == 'set' : setValue = True
            elif stepType == 'action' and element.type in [int,float,str] : setValue = True
            else : setValue = False
            
            if setValue is True :
                if value is None : 
                    if element.type in [int,float] :
                        value = 0
                    elif element.type in [str] :
                        value = ''
                    elif element.type in [bool]:
                        value = False
                step['value'] = value
            
            self.config['recipe'].append(step)
            self.gui.recipeManager.refresh()
            self.gui.dataManager.clear()
            
            
            
    def delRecipeStep(self,name):
        
        """ This function removes a step from the scan recipe """
        
        if self.gui.scanManager.isStarted() is False :
            pos = self.getRecipeStepPosition(name)
            self.config['recipe'].pop(pos)
            self.gui.recipeManager.refresh()
            self.gui.dataManager.clear()
            
            
    
    def renameRecipeStep(self,name,newName):
        
        """ This function rename a step in the scan recipe """
        
        if self.gui.scanManager.isStarted() is False :
            if newName != name :
                pos = self.getRecipeStepPosition(name)
                newName = self.getUniqueName(newName)
                self.config['recipe'][pos]['name'] = newName
                self.gui.recipeManager.refresh()
                self.gui.dataManager.clear()
                
                
                
    def setRecipeStepValue(self,name,value):
        
        """ This function set the value of a step in the scan recipe """
        
        if self.gui.scanManager.isStarted() is False :
            pos = self.getRecipeStepPosition(name)
            if value != self.config['recipe'][pos]['value'] :
                self.config['recipe'][pos]['value'] = value
                self.gui.recipeManager.refresh()
                self.gui.dataManager.clear()    
                
                
                
    def setRecipeOrder(self,stepOrder):
        
        """ This function reorder the recipe as a function of the list of step names provided """
        
        newOrder = [self.getRecipeStepPosition(name) for name in stepOrder]
        recipe = self.config['recipe']
        self.config['recipe'] = [recipe[i] for i in newOrder]
        self.gui.recipeManager.refresh()
        self.gui.dataManager.clear()  
                
        
                
    def setNbPts(self,value):
        
        """ This function set the value of the number of points of the scan """
        
        if self.gui.scanManager.isStarted() is False :
            self.config['nbpts'] = value
        self.gui.rangeManager.refresh()
        
        
        
    def setRange(self,lim):
        
        """ This function set the range (start and end value) of the scan """
        
        if self.gui.scanManager.isStarted() is False :
            if lim != self.config['range'] :
                self.config['range'] = tuple(lim)
        self.gui.rangeManager.refresh()
        
        
        
    def setLog(self,state):
        
        """ This function set the log state of the scan """
        
        if self.gui.scanManager.isStarted() is False :
            if state != self.config['log']:
                self.config['log'] = state
        self.gui.rangeManager.refresh()
                
            
            
            
    # CONFIG READING
    ###########################################################################

    def getParameter(self):
        
        """ This function returns the element of the current parameter of the scan """
        
        return self.config['parameter']['element']
    
    
    
    def getParameterName(self):
        
        """ This function returns the name of the current parameter of the scan """
        
        return self.config['parameter']['name']
    
    
    
    def getRecipeStepElement(self,name):
        
        """ This function returns the element of a recipe step """
        
        pos =  self.getRecipeStepPosition(name)
        return self.config['recipe'][pos]['element']
            
    
    
    def getRecipeStepType(self,name):
        
        """ This function returns the type a recipe step """

        pos =  self.getRecipeStepPosition(name)
        return self.config['recipe'][pos]['stepType']
    
    
    
    def getRecipeStepValue(self,name):
        
        """ This function returns the value of a recipe step """
        
        pos =  self.getRecipeStepPosition(name)
        return self.config['recipe'][pos]['value']
    
    
    
    def getRecipeStepPosition(self,name):
        
        """ This function returns the position of a recipe step in the recipe """
        
        return [i for i, step in enumerate(self.config['recipe']) if step['name'] == name][0]     
    
    
        
    def getLog(self):
        
        """ This function returns the log state of the scan """
        
        return self.config['log']
    
    
    
    def getNbPts(self):
        
        """ This function returns the number of points of the scan """
        
        return self.config['nbpts']
    
    
    
    def getRange(self):
        
        """ This function returns the range (start and end value) of the scan """
        
        return self.config['range']
    
    
    
    def getRecipe(self):
        
        """ This function returns the whole recipe of the scan """
        
        return self.config['recipe']
    
    
                        
    def getConfig(self):
        
        """ This function returns the whole configuration of the scan """
        
        return self.config
  
    
    
    

    # EXPORT IMPORT ACTIONS
    ###########################################################################

    def exportActionClicked(self):
        
        """ This function prompts the user for a configuration file path, 
        and export the current scan configuration in it """

        path = QtWidgets.QFileDialog.getSaveFileName(self.gui, "Export AUTOLAB configuration file", 
                                                     os.path.join(paths.USER_LAST_CUSTOM_FOLDER,'config.conf'), 
                                                     "AUTOLAB configuration file (*.conf)")[0]

        if path != '' :
            paths.USER_LAST_CUSTOM_FOLDER = path
            
            try : 
                self.export(path)
                self.gui.statusBar.showMessage(f"Current configuration successfully saved at {path}",5000)
            except Exception as e :
                self.gui.statusBar.showMessage(f"An error occured: {str(e)}",10000)
                
                
         
    
    
    def export(self,path):
        
        """ This function exports the curr scan configuration in the provided path """
        
        configPars = configparser.ConfigParser()
        configPars['autolab'] = {'version':autolab.__version__,
                                  'timestamp':str(datetime.datetime.now())}
                
        configPars['parameter'] = {}
        if self.config['parameter']['element'] is not None :
            configPars['parameter']['name'] = self.config['parameter']['name']
            configPars['parameter']['address'] = self.config['parameter']['element'].address()
        configPars['parameter']['nbpts'] = str(self.config['nbpts'])
        configPars['parameter']['start_value'] = str(self.config['range'][0])
        configPars['parameter']['end_value'] = str(self.config['range'][1])
        configPars['parameter']['log'] = str(int(self.config['log']))
        
        configPars['recipe'] = {}
        for i in range(len(self.config['recipe'])) :
            configPars['recipe'][f'{i+1}_name'] = self.config['recipe'][i]['name']
            configPars['recipe'][f'{i+1}_stepType'] = self.config['recipe'][i]['stepType']
            configPars['recipe'][f'{i+1}_address'] = self.config['recipe'][i]['element'].address()
            stepType = self.config['recipe'][i]['stepType']
            if stepType == 'set' or (stepType == 'action' and self.config['recipe'][i]['element'].type in [int,float,str]) :
                value = self.config['recipe'][i]['value']
                if self.config['recipe'][i]['element'].type in [str] :
                    valueStr = f'{value}'
                else :
                    valueStr = f'{value:g}'
                configPars['recipe'][f'{i+1}_value'] = valueStr
                
        with open(path, 'w') as configfile:
            configPars.write(configfile)
            
            
        
    def importActionClicked(self):
        
        """ This function prompts the user for a configuration file path, 
        and import the current scan configuration from it """
        
        path = QtWidgets.QFileDialog.getOpenFileName(self.gui, "Import AUTOLAB configuration file", 
                                                     paths.USER_LAST_CUSTOM_FOLDER,
                                                     "AUTOLAB configuration file (*.conf)")[0]
        if path != '' :
            
            configPars = configparser.ConfigParser()
            configPars.read(path)
            
            try :
            
                assert 'parameter' in configPars
                config = {}
                
                if 'address' in configPars['parameter'] :
                    element = autolab.get_element_by_address(configPars['parameter']['address'])
                    assert element is not None, f"Parameter {configPars['parameter']['address']} not found."
                    assert 'name' in configPars['parameter'], f"Parameter name not found."
                    config['parameter'] = {'element':element,'name':configPars['parameter']['name']}
                    
                
                assert 'recipe' in configPars
                for key in ['nbpts','start_value','end_value','log'] :
                    assert key in configPars['parameter'], "Missing parameter key {key}."
                    config['nbpts'] = int(configPars['parameter']['nbpts'])
                    start = float(configPars['parameter']['start_value'])
                    end = float(configPars['parameter']['end_value'])
                    config['range'] = (start,end)
                    config['log'] = bool(int(configPars['parameter']['log']))                    
                
                
                config['recipe'] = []

                while True :
                    step = {}
                    
                    i = len(config['recipe'])+1
    
                    if f'{i}_name' in configPars['recipe']:
                        
                        step['name'] = configPars['recipe'][f'{i}_name']
                        name = step['name']
                        
                        assert f'{i}_stepType' in configPars['recipe'], f"Missing stepType in step {i} ({name})."
                        step['stepType'] = configPars['recipe'][f'{i}_stepType']
                        
                        assert f'{i}_address' in configPars['recipe'], f"Missing address in step {i} ({name})."
                        address = configPars['recipe'][f'{i}_address']
                        element = autolab.get_element_by_address(address)
                        assert element is not None, f"Address {address} not found for step {i} ({name})."
                        step['element'] = element
                        
                        if step['stepType']=='set' or (step['stepType'] == 'action' and element.type in [int,float,str]) :
                            assert f'{i}_value' in configPars['recipe'], f"Missing value in step {i} ({name})."
                            value = configPars['recipe'][f'{i}_value']
                            if element.type in [int]:
                                value = int(value)
                            elif element.type in [float] :
                                value = float(value)
                            elif element.type in [str] :
                                value = str(value)
                            elif element.type in [bool]:
                                value = int(value)
                                assert value in [0,1]
                                value = bool(value)
                            step['value'] = value
                        else :
                            step['value'] = None
                                                        
                        config['recipe'].append(step)
                        
    
                    else :
                        break
                
                
                self.config = config
                
                self.gui.dataManager.clear()
                self.gui.parameterManager.refresh()
                self.gui.recipeManager.refresh()
                self.gui.rangeManager.refresh()
                
                self.gui.statusBar.showMessage(f"Configuration file loaded successfully",5000)
            
            except Exception as e :
                
                self.gui.statusBar.showMessage(f"Impossible to load configuration file: "+str(e),10000)
            


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Oscilloscope'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Extra infos:  ----------------
    Datas are recorded in int8 format
    
    Headers contains:
    :WFMOUTPRE:BYT_NR 2;BIT_NR 16;ENCDG ASCII;BN_FMT RI;BYT_OR
    MSB;WFID 'Ch1, DC coupling, 100.0mV/div, 4.000us/div, 10000
    points, Sample mode';NR_PT 10000;PT_FMT Y;XUNIT 's';XINCR
    4.0000E-9;XZERO - 20.0000E-6;PT_OFF 0;YUNIT 'V';YMULT
    15.6250E-6;YOFF :”6.4000E+3;YZERO 0.0000
    
    To retrieve real value:
    value_in_units = ((curve_in_dl - YOFf_in_dl) * YMUlt) + YZEro_in_units

----------------  Examples:  ----------------

usage:    autolab driver [options] args
        
    autolab driver -D {self.name} -A 192.168.0.3 -C VXI11 -o my_output_file -c 1
    Results in saving two files for the temporal trace of channel 1, the data and the scope parameters, called respectively my_output_file_TDS5104BCH1 and my_output_file_TDS5104BCH1.log
    
    autolab driver -D nickname -o my_output_file -F -c 1,2
    Same as previous one but with 4 output files, two for each channel (1 and 2) and using the device nickname as defined in local_config.ini. If files with your filename already exist -F flag overwrite them.
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from." )
        parser.add_argument("-o", "--filename", type=str, dest="filename", default=None, help="Set the name of the output file" )
        parser.add_argument("-F", "--force",action="store_true", dest="force", default=None, help="Allows overwriting file" )
        
        return parser

    def do_something(self,args):
        if args.filename:
            getattr(self.Instance,'get_data_channels')(channels=args.channels.split(','))
            getattr(self.Instance,'save_data_channels')(filename=args.filename,channels=args.channels.split(','),FORCE=args.force)
  

    def exit(self):
        self.Instance.close()




# -*- coding: utf-8 -*-
"""
Created on Fri May 17 15:04:04 2019

@author: quentin.chateiller
"""
import os

# VERSION
with open(os.path.join(os.path.dirname(__file__), 'version.txt')) as version_file:
    __version__ = version_file.read().strip()
del version_file

# PATHS
from .core import paths

# CONFIG
from .core import stats as _stats
from .core import config as _config
_config.check()

# STATS
from .core.stats import set_stats_enabled, is_stats_enabled
_stats.startup()

# DRIVERS
from .core.drivers import *

# DEVICES
from .core.devices import *

# WEBBROWSER FUNCTIONS
from .core.web import *

# RECORDER (to be removed at some point)
from .core.recorder import Recorder, Recorder_V2

# GUI
from .core.gui import start as gui





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    def __init__(self,nb_channels=4):
        
        self.nb_channels = int(nb_channels)
        
        for i in range(1,self.nb_channels+1):
            setattr(self,f'channel{i}',Channel(self,i))
    
    
    def write_to_channel(self,channel,command):
        self.write(f'SETUPCH {channel}')
        self.write(command)
        
        
    def idn(self):
        self.write('*IDN?\r\n')
        return self.read()
        
    def get_driver_model(self):
        model = []
        for num in range(1,self.nb_channels+1) :
            model.append({'element':'module','name':f'channel{num}','object':getattr(self,f'channel{num}')})
        return model

#################################################################################
############################## Connections classes ##############################
class Driver_USB(Driver):
    def __init__(self, **kwargs):
        import usb
        import usb.core
        import usb.util
        
        dev = usb.core.find(idVendor=0x103e,idProduct=0x03f2)
        dev.reset()
        dev.set_configuration()

        interface = 0
        if dev.is_kernel_driver_active(interface) is True:
            # tell the kernel to detach
            dev.detach_kernel_driver(interface)
            # claim the device
            usb.util.claim_interface(dev, interface)

        cfg = dev.get_active_configuration()
        intf = cfg[(0,0)]
        self.ep_out = usb.util.find_descriptor(intf,custom_match = lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_OUT)
        self.ep_in = usb.util.find_descriptor(intf,custom_match = lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_IN)

        assert self.ep_out is not None
        assert self.ep_in is not None
        rm        = v.ResourceManager()
        self.inst = rm.get_instrument(address)
        
        Driver.__init__(self, **kwargs)
        
    def close(self):
        self.write('LOCAL')
        pass
    def write(self,query):
        self.string = query + '\r\n'
        self.ep_out.write(self.string)
    def read(self):
        rep = self.ep_in.read(2000000)
        const = ''.join(chr(i) for i in rep)
        const = const#[:const.find('\r\n')]
        return const
        
############################## Connections classes ##############################
#################################################################################


class Channel():
    def __init__(self,dev,channel):
        self.channel = int(channel)
        self.dev     = dev

    def amplitude(self,val):
        """ Change amplitude in Vpp """
        self.dev.write_to_channel(self.channel,f'AMPL {val}')
    def frequency(self,val):
        """ Change the frequency in Hz """
        self.dev.write_to_channel(self.channel,f'WAVFREQ {val}')
    def period(self,val):
        """ Change the period in s """
        self.dev.write_to_channel(self.channel,f'WAVPER {val}')


    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'amplitude','write':self.amplitude,'type':float,'help':"set the amplitude"})
        model.append({'element':'variable','name':'period','write':self.period,'type':float,'help':"Set the period"})
        model.append({'element':'variable','name':'frequency','write':self.frequency,'type':float,'help':"Set the frequency"})
        return model



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- Exfo pm1613 
"""
import time

class Driver():
    
    def __init__(self):
        
        # Initialisation
        self.write('*CLS')
        self.write('SENS:POW:RANG:AUTO 1')      # Ajuster la gamme de mesure automatiquement
        self.write('SENS:POW:REF:STAT 0')       # Set Absolute power measurment mode (dBm or W)
        self.write('SENS:POW W')                # Unité = Watts
        self.write('SENS:POW:UNIT W')
        
        
    def get_id(self):
        return self.query('*IDN?')
    
    def set_averaging_state(self,state):
        assert isinstance(state,bool)
        currentState=self.get_averaging_state()
        if state != currentState :
            self.write('SENS:AVER:STAT %i'%int(state))
            self.query('*OPC?')
    
    def get_averaging_state(self):
        return bool(self.query('SENS:AVER:STAT?'))
        
    def zero(self):
        self.write('SENS:CORR:COLL:ZERO')
        self.query('*OPC?')
    
    def get_buffer_size(self):
        return int(self.query('SENS:AVER:COUN?'))
    
    def set_buffer_size(self, value):
        assert isinstance(int(value),int)
        value=int(value)
        current_size=self.get_buffer_size()
        if current_size != value :
            self.write('SENS:AVER:COUN %i'%value)
            self.query('*OPC?')
        
    def get_power(self):
        while True :
            result=self.query('READ:ALL:POW:DC?')
            if isinstance(result,str) and '*' in result :
                time.sleep(0.1)
            else :
                break
        return float(result)

    def set_wavelength(self,wavelength):
        assert isinstance(float(wavelength),float)
        wavelength=float(wavelength)
        current_wavelength=self.get_wavelength()
        if wavelength != current_wavelength :
            self.write('SENS:POW:WAVE %f'%wavelength)
            self.query('*OPC?')
    
    def get_wavelength(self):
        return float(self.query('SENS:POW:WAVE?'))
    
    
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'averaging','write':self.set_averaging_state,'read':self.get_averaging_state,'type':bool,'help':'This command activates or deactivates data averaging.'})
        model.append({'element':'variable','name':'buffer_size','write':self.set_buffer_size,'read':self.get_buffer_size,'type':int,'help':'This command sets the number of power measurements that will be used to compute data averaging.'})
        model.append({'element':'variable','name':'wavelength','write':self.set_wavelength,'read':self.get_wavelength,'type':float,'help':'The <numeric_value> parameter is an operating wavelength in nm. Any wavelength within the spectral range of the power meter optical detector at 0.01 nm resolution may be selected.'})
        model.append({'element':'variable','name':'power','read':self.get_power,'type':float,'help':'This command returns the power of both channels in their respective current unit.'})
        model.append({'element':'action','name':'zero','do':self.zero, 'help':'This command performs an offset nulling measurement.'})       
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        import visa
        
        self.TIMEOUT = 10000 #ms
        
        # Instantiation
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.timeout = self.TIMEOUT
        Driver.__init__(self)        

    def close(self):
        try : self.controller.close()
        except : pass

    def query(self,command):
        return self.controller.query(command).strip('\n')
    
    def write(self,command):
        self.controller.write(command)
        
        
############################## Connections classes ##############################
#################################################################################
        
        



# -*- coding: utf-8 -*-
"""
Created on Tue Oct  1 17:38:15 2019

@author: qchat
"""
import os 
import inspect
import importlib
import autolab
import configparser
from .utilities import emphasize,underline,two_columns

    
# =============================================================================
# DRIVERS INSTANTIATION
# =============================================================================

def get_driver(name, **kwargs):
    
    ''' Returns a driver instance '''
    
    if name in list_local_configs() :
        
        # Load config object
        config = dict(get_local_config(name))
        
        # Overwrite config with provided configuration in kwargs
        for key,value in kwargs.items() :
            config[key] = value
        
        # Check if driver provided
        assert 'driver' in config.keys(), f"Driver name not found in driver config '{name}'"
        driver_name = config['driver']
        del config['driver'] # Because config will be pass to the Driver class
            
    else :
        
        # Create config object
        config = kwargs
    
        # The provided name has to be a driver name
        assert name in list_drivers(), f'Driver {name} not found'
        driver_name = name
        
    # And the argument connection has to be provided
    assert 'connection' in config.keys(), f"Missing connection type for driver '{driver_name}'"
    connection = config['connection']
    del config['connection'] # Because config will be pass to the Driver class
        
    # Load driver library
    driver_lib = load_driver_lib(driver_name)
        
    # Check connection type
    assert connection in get_connection_names(driver_lib),f"Connection type {connection} not found for driver {driver_name}"
    instance = get_connection_class(driver_lib,connection)(**config)
    
    return instance
    


def load_driver_lib(driver_name):
    
    ''' Returns a driver library (that contains Driver, Driver_XXX, Module_XXX) '''
            
    # Loading preparation
    driver_path = get_driver_path(driver_name)
    
    # Laod library
    driver_lib = load_lib(driver_path)
    
    return driver_lib




def load_lib(lib_path):
    
    ''' Return an instance of the python script located at lib_path '''
    
    lib_name = os.path.basename(lib_path).split('.')[0]
    
    # Save current working directory path
    curr_dir = os.getcwd()
    
    # Go to the driver's directory (in case it contains absolute imports)
    os.chdir(os.path.dirname(lib_path))

    # Load the module
    spec = importlib.util.spec_from_file_location(lib_name, lib_path)
    lib = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(lib)
    
    # Come back to previous working directory
    os.chdir(curr_dir)
    
    return lib
    
    
    
    
def load_driver_utilities_lib(driver_utilities_name):
    
    ''' Returns a driver library (that contains Driver, Driver_XXX, Module_XXX) '''
            
    # Loading preparation
    driver_path = get_driver_path(driver_utilities_name.replace('_utilities',''))
    
    # Laod library
    driver_lib = load_utilities_lib(driver_path)
    
    return driver_lib




def load_utilities_lib(lib_path):
    
    ''' Return an instance of the python script located at lib_path '''
    
    lib_name = os.path.basename(lib_path).split('.')[0]
    
    # Save current working directory path
    curr_dir = os.getcwd()
    
    # Go to the driver's directory (in case it contains absolute imports)
    os.chdir(os.path.dirname(lib_path))

    # Load the module
    lib_name = lib_name + '_utilities'
    spec = importlib.util.spec_from_file_location(lib_name, os.path.join(os.path.dirname(lib_path),f'{lib_name}.py'))
    lib = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(lib)
    
    # Come back to previous working directory
    os.chdir(curr_dir)
    
    return lib

# =============================================================================
# DRIVERS LIST HELP
# =============================================================================

def list_drivers():
    
    ''' Returns the list of available drivers '''
    
    return sorted(list(DRIVERS_PATHS.keys()))


    
    
    
    
    

# =============================================================================
# DRIVERS INSPECTION
# =============================================================================

def get_module_names(driver_lib):
    
    ''' Returns the list of the driver's Module(s) name(s) (classes Module_XXX) '''
    
    return [name.split('_')[1] 
            for name, obj in inspect.getmembers(driver_lib, inspect.isclass) 
            if obj.__module__ is driver_lib.__name__ 
            and name.startswith('Module_')]
    
    
            
def get_connection_names(driver_lib):
    
    ''' Returns the list of the driver's connection types (classes Driver_XXX) '''
    
    return [name.split('_')[1]
            for name, obj in inspect.getmembers(driver_lib, inspect.isclass) 
            if obj.__module__ is driver_lib.__name__ 
            and name.startswith('Driver_')]
    
    
    
def get_driver_category(driver_name):
    
    ''' Returns the driver's category (from class Driver) '''
    
    driver_utilities_path = os.path.join(os.path.dirname(DRIVERS_PATHS[driver_name]),f'{driver_name}_utilities.py')
    category = 'Other'
    if os.path.exists(driver_utilities_path) :
        driver_utilities = load_lib(driver_utilities_path)
        if hasattr(driver_utilities,'category') :
            category = driver_utilities.category
    
    return category
    
    
    
def get_driver_class(driver_lib):
    
    ''' Returns the class Driver of the provided driver library '''
    
    assert hasattr(driver_lib,'Driver'), f"Class Driver missing in driver {driver_lib.__name__}"
    assert inspect.isclass(driver_lib.Driver), f"Class Driver missing in driver {driver_lib.__name__}"
    return driver_lib.Driver
    


def get_connection_class(driver_lib,connection):
    
    ''' Returns the class Driver_XXX of the provided driver library and connection type '''
    
    assert connection in get_connection_names(driver_lib)
    return getattr(driver_lib,f'Driver_{connection}')



def get_module_class(driver_lib,module_name):
    
    ''' Returns the class Module_XXX of the provided driver library and module_name'''
    
    assert module_name in get_module_names(driver_lib)
    return getattr(driver_lib,f'Module_{module_name}')


def explore_driver(instance,_print=True):
    
    ''' Displays the list of the methods available in this instance '''
    
    methods = get_instance_methods(instance)
    s = 'This instance contains the following functions:\n'
    for method in methods :
        s += f' - {method[0]}({",".join(method[1])})\n'
        
    if _print is True : print(s)
    else : return s


def get_instance_methods(instance):
    
    ''' Returns the list of all the methods (and their args) in that class '''
    
    methods = []
    
    # LEVEL 1
    for name,obj in inspect.getmembers(instance,inspect.ismethod) :
        if name != '__init__' :
            attr = getattr(instance,name)
            args = list(inspect.signature(attr).parameters.keys())
            methods.append([name,args])
            
    # LEVEL 2
    instance_vars = vars(instance)
    for key in instance_vars.keys():
        try :    # explicit to avoid visa and inspect.getmembers issue
            for name,obj in inspect.getmembers(instance_vars[key],inspect.ismethod):
                if inspect.getmembers(instance_vars[key],inspect.ismethod) != '__init__' and inspect.getmembers(instance_vars[key],inspect.ismethod) and name!='__init__':
                    attr = getattr(getattr(instance,key),name)
                    args = list(inspect.signature(attr).parameters.keys())
                    methods.append([f'{key}.{name}',args])
        except : pass
        
    return methods


def get_class_args(clas):
    
    ''' Returns the dictionary of the optional arguments required by a class
    with their default values '''
    
    signature = inspect.signature(clas)
    return {k: v.default for k, v in signature.parameters.items() if v.default is not inspect.Parameter.empty}




# =============================================================================
# LOCAL CONFIGS
# =============================================================================
    
def load_local_configs():
    
    ''' Return the current autolab drivers informations:
        - the content of the local_config file  ['config']
    '''
    
    paths = autolab.paths
    
    config = configparser.ConfigParser()
    config.read(paths.LOCAL_CONFIG)
    assert len(set(config.sections())) == len(config.sections()), f"Each device must have a unique name."
    
    return config



def list_local_configs():
    
    ''' Returns the list of available configuration names '''
    
    local_configs = load_local_configs()
    return sorted(list(local_configs.sections()))



def get_local_config(config_name):
    
    ''' Returns the config associated with config_name '''
    
    assert config_name in list_local_configs(), f"Configuration {config_name} not found"
    local_configs = load_local_configs()
    return local_configs[config_name]



def set_local_config(config_name,modify=False,**kwargs):
    
    ''' Add a new driver config (kwargs) or modify an existing one in the configuration file.
    Set the option modify=True to apply changes in an existing config (for safety) '''
    
    if modify is False :
        assert config_name not in list_local_configs(), f'Configuration "{config_name}" already exists. Please set the option modify=True to apply changes, or delete the configuration first.'
    
    local_configs = load_local_configs()
    
    if config_name not in local_configs.sections():
        local_configs.add_section(config_name)
    
    for key,value in kwargs.items() :
        local_configs.set(config_name, key, str(value))
        
    for key in ['driver', 'connection'] :
        assert key in local_configs[config_name].keys(), f'Missing parameter "{key}" in the configuration'
    assert local_configs.get(config_name, 'driver') in list_drivers(), f'Driver "{local_configs.get(config_name, "driver")}" does not exist'
        
    with open(autolab.paths.LOCAL_CONFIG, 'w') as file:
        local_configs.write(file)
     


def show_local_config(config_name,_print=True):
    
    ''' Display a local_config as it appears in the configuration file '''
    
    assert config_name in list_local_configs(), f'Configuration "{config_name}" does not exist.'
    
    local_configs = load_local_configs()
    mess = f'\n[{config_name}]\n'
    for key,value in local_configs[config_name].items() :
        mess += f'{key} = {value}\n'
    
    if _print is True : print(mess)
    else : return mess

        
        
def remove_local_config(config_name):
    
    ''' Remove a local_config from the configuration file '''
    
    assert config_name in list_local_configs(), f'Configuration "{config_name}" does not exist.'
    
    local_configs = load_local_configs()
    local_configs.remove_section(config_name)

    with open(autolab.paths.LOCAL_CONFIG, 'w') as file:
        local_configs.write(file)



def remove_local_config_parameter(config_name,param_name):
    
    ''' Remove a parameter of a local_config in the configuration file '''
    
    assert config_name in list_local_configs(), f'Configuration "{config_name}" does not exist.'
    
    local_configs = load_local_configs()
    assert param_name in local_configs[config_name].keys(), f'The parameter "{param_name}" does not exist.'
    assert param_name not in ['driver','connection'], f'The parameter "{param_name}" is mandatory.'
    
    local_configs = load_local_configs()
    local_configs.remove_option(config_name,param_name)
    
    with open(autolab.paths.LOCAL_CONFIG, 'w') as file:
        local_configs.write(file)
    

def config_help(driver_name, _print=True, _parser=False):
    
    ''' Display the help of a particular driver (connection types, modules, ...) '''
    
    
    # Load list of all parameters
    driver_lib = load_driver_lib(driver_name)
    params = {}
    params['driver'] = driver_name
    params['connection'] = {}
    for conn in get_connection_names(driver_lib) : 
        params['connection'][conn] = get_class_args(get_connection_class(driver_lib,conn))
    params['other'] = get_class_args(get_driver_class(driver_lib))
    if hasattr(get_driver_class(driver_lib),'slot_config') :
        params['other']['slot1'] = f'{get_driver_class(driver_lib).slot_config}'
        params['other']['slot1_name'] = 'my_<MODULE_NAME>'
    
    mess = '\n'

    # Name and category if available
    submess = f'Driver "{driver_name}" ({get_driver_category(driver_name)})'
    mess += emphasize(submess,sign='=') + '\n'
    
    # Connections types
    c_option=''
    if _parser: c_option='(-C option)'
    mess += f'\nAvailable connections types {c_option}:\n'
    for connection in params['connection'].keys() : 
        mess += f' - {connection}\n'
    mess += '\n'
    
    # Modules
    if hasattr(get_driver_class(driver_lib),'slot_config') :
        mess += 'Available modules:\n'
        modules = get_module_names(driver_lib)
        for module in modules : 
            moduleClass = get_module_class(driver_lib,module)
            mess += f' - {module}'
            if hasattr(moduleClass,'category') : mess += f' ({moduleClass.category})'
            mess += '\n'
        mess += '\n'
   
    # Example of get_driver
    mess += '\n' + underline('Example(s) to load a Driver or a Device:') + '\n\n'
    for conn in params['connection'].keys() :
        if _parser is False :
            args_str = f"'{params['driver']}', connection='{conn}'"
            for arg,value in params['connection'][conn].items():
                args_str += f", {arg}='{value}'"
            for arg,value in params['other'].items():
                args_str += f", {arg}='{value}'"
            mess += f"   a = autolab.get_driver({args_str})\n"
            mess += f"   a = autolab.get_device({args_str})\n"
        else : 
            args_str = f"-D {params['driver']} -C {conn} "
            for arg,value in params['connection'][conn].items():
                if arg == 'address' : args_str += f"-A {value} "
                if arg == 'port' : args_str += f"-P {value} "
            if len(params['other'])>0 : args_str += '-O '
            for arg,value in params['other'].items():
                args_str += f"{arg}={value} "
            mess += f"   autolab driver {args_str} \n"
            mess += f"   autolab device {args_str} \n"
            
    # Example of set_local_config
    mess += '\n\n' + underline('Example(s) to save a configuration by command-line:') + '\n\n'
    for conn in params['connection'].keys() :
        if _parser is False :
            args_str = f"'my_{params['driver']}', driver='{params['driver']}', connection='{conn}'"
            for arg,value in params['connection'][conn].items():
                args_str += f", {arg}='{value}'"
            for arg,value in params['other'].items():
                args_str += f", {arg}='{value}'"
            mess += f"   autolab.set_local_config({args_str})\n"
        else : 
            mess += '   Available soon through OS shell\n'
            
    # Example of set_local_config
    mess += '\n\n' + underline('Example(s) to save a configuration by editing the file local_config.ini:') + '\n'
    for conn in params['connection'].keys() :
        mess += f"\n   [my_{params['driver']}]\n"
        mess += f"   driver = {params['driver']}\n"
        mess += f"   connection = {conn}\n"
        for arg,value in params['connection'][conn].items():
            mess += f"   {arg} = {value}\n"
        for arg,value in params['other'].items():
            mess += f"   {arg} = {value}\n"
    
    # Example of get_driver_by_config
    mess += '\n\n' + underline('Example to instantiate a Driver or a Device with a local configuration:') + '\n\n'
    if _parser is False :
        mess += f"   a = autolab.get_driver('my_{params['driver']}')\n"
        mess += f"   a = autolab.get_device('my_{params['driver']}')"
    else :
        mess += f"   autolab driver -D my_{params['driver']}\n"
        mess += f"   autolab device -D my_{params['driver']}\n"
        
    if _print is True : print(mess)
    else : return mess
    




# =============================================================================
# DRIVERS PATHS
# =============================================================================

def get_driver_path(driver_name):
    
    ''' Returns the config associated with driver_name '''
    
    assert driver_name in DRIVERS_PATHS.keys(), f'Driver {driver_name} not found.'
    return DRIVERS_PATHS[driver_name]

def load_drivers_paths():
    
    ''' Returns a dictionary with :
        - key : name of the driver
        - value : path of the driver python script
    '''
    
    paths = autolab.paths
    
    drivers_paths = {}
    for driver_source in paths.DRIVER_SOURCES.values():
        for name in os.listdir(driver_source) :
            temp_path = os.path.join(driver_source,name)
            if os.path.isdir(temp_path) and f'{name}.py' in os.listdir(temp_path) :
                assert name not in drivers_paths.keys(), f"Two drivers where found with the name '{name}'. Each driver must have a unique name."
                drivers_paths[name] = os.path.join(temp_path,f'{name}.py')
              
    return drivers_paths






# =============================================================================
# INFOS
# =============================================================================

def infos(_print=True):
    
    # Gather drivers informations
    drivers = {}
    for driver_name in DRIVERS_PATHS.keys() :
        driver_source = os.path.dirname(os.path.dirname(DRIVERS_PATHS[driver_name]))
        if driver_source not in drivers.keys() : drivers[driver_source] = []
        drivers[driver_source].append(driver_name)
    
    # Gather local config informations
    local_configs = list_local_configs()
    
    # Print infos
    s = '\n'
    s += f'{len(DRIVERS_PATHS)} drivers found\n'
    s += f'{len(local_configs)} local configurations found\n\n'
    for driver_source in drivers.keys() : 
        s += f'Drivers in {driver_source}:\n'
        txt_list = [[f'    - {driver_name}',f'({get_driver_category(driver_name)})']
                    for driver_name in drivers[driver_source] ]
        s += two_columns(txt_list)+'\n\n'
        
    s += 'Local configurations:\n'
    txt_list = [[f'    - {config_name}',f'({get_local_config(config_name)["driver"]})']
                for config_name in local_configs ]
    s += two_columns(txt_list)+'\n'
    
    if _print is True : print(s)
    else : return s
    
    


# Loading the drivers informations at startup
DRIVERS_PATHS = load_drivers_paths()
        



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 13 23:29:33 2017

@author: Quentin Chateiller
quentin.chateiller@c2n.upsaclay.fr

"""

import autolab
from PyQt5 import QtCore, QtWidgets, uic, QtGui
import os 
from ..scanning.main import Scanner

from .thread import ThreadManager
from .treewidgets import TreeWidgetItemModule


class ControlCenter(QtWidgets.QMainWindow):
        
    def __init__(self):
                
        # Set up the user interface from Designer.
        QtWidgets.QMainWindow.__init__(self)
        ui_path = os.path.join(os.path.dirname(__file__),'interface.ui')
        uic.loadUi(ui_path,self)
                
        # Window configuration
        self.setWindowTitle("AUTOLAB - Control Panel")
        self.setFocus()
        self.activateWindow()
        
        # Tree widget configuration
        self.tree.setHeaderLabels(['Objects','Type','Actions','Values',''])
        self.tree.header().setDefaultAlignment(QtCore.Qt.AlignCenter);
        self.tree.header().resizeSection(0, 200)
        self.tree.header().resizeSection(4, 15)
        self.tree.header().setStretchLastSection(False)
        self.tree.itemClicked.connect(self.itemClicked)
        self.tree.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tree.customContextMenuRequested.connect(self.rightClick)
        self.tree.setAlternatingRowColors(True)
        
        # Thread manager
        self.threadManager = ThreadManager(self)
        
        # Scanner / Monitors
        self.scanner = None
        self.monitors = {}
        
        
        scanAction = self.menuBar.addAction('Open scanner')
        scanAction.triggered.connect(self.openScanner)
        scanAction.setToolTip('Open the scanner in another window')  

        
        reportAction = self.menuBar.addAction('Report bugs / suggestions')
        reportAction.triggered.connect(autolab.report)
        reportAction.setToolTip('Open the issue webpage of this project on GitHub')  
        
        helpAction = self.menuBar.addAction('Help')
        helpAction.triggered.connect(autolab.doc)
        helpAction.setToolTip('Open the documentation on Read The Docs website')  
        
    def initialize(self):
        
        """ This function will create the first items in the tree, but will 
        associate only the ones already loaded in autolab """
        
        for devName in autolab.list_local_configs() :
            item = TreeWidgetItemModule(self.tree,devName,self)
            for i in range(5) :
                item.setBackground(i, QtGui.QColor('#9EB7F5')) #vert
            if devName in autolab.list_devices() :
                self.associate(item)
        
        
        
    def setStatus(self,message):
    
        """ Modify the message displayed in the status bar """
        
        self.statusbar.showMessage(message)
        
        
        
    def clearStatus(self):
        
        """ Erase the message displayed in the status bar """
        
        self.setStatus('')
        
        
    
    def rightClick(self,position):
        
        """ Function called when a right click has been detected in the tree """
        
        item = self.tree.itemAt(position)
        if hasattr(item,'menu') :
            item.menu(position)
        
        

        
        

    def itemClicked(self,item):
                
        """ Function called when a normal click has been detected in the tree.
            Check the association if it is a main item """
        
        if item.parent() is None and item.loaded is False :
            self.associate(item)
            item.setExpanded(True)
        
        
        
    def associate(self,item):
        
        """ Function called to associate a main module to one item in the tree """
        
        # Try to get / instantiated the device
        check = False
        try : 
            self.setStatus(f'Loading device {item.name}...')
            module = autolab.get_device(item.name)
            check = True
            self.clearStatus()
        except Exception as e : 
            self.setStatus(f'An error occured when loading device {item.name} : {str(e)}')
            
        # If success, load the entire module (submodules, variables, actions)
        if check is True : 
            item.load(module)
            
                    
                
            
            

    def openScanner(self):
        
        """ This function open the scanner associated to this variable. """
        
        # If the scanner is not already running, create one
        if self.scanner is None : 
            self.scanner = Scanner(self)
            self.scanner.show()
            self.scanner.activateWindow()
            self.activateWindow() # Put main window back to the front

        
        # If the scanner is already running, just make as the front window
        else :  
            self.scanner.setWindowState(self.scanner.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
            self.scanner.activateWindow()
             
            
            
    def setScanParameter(self,variable):
        
        if self.scanner is None : 
            self.openScanner()
            
        self.scanner.configManager.setParameter(variable)
        
        
    def addStepToScanRecipe(self,stepType,element):
        
        if self.scanner is None : 
            self.openScanner()
        
        self.scanner.configManager.addRecipeStep(stepType,element)
                
                    
    def clearScanner(self):
        
        """ This clear the gui instance reference when quitted """
        
        self.scanner = None
        
        
    def closeEvent(self,event):
        
        """ This function does some steps before the window is really killed """
        
        if self.scanner is not None :
            self.scanner.close()
            
        monitors = list(self.monitors.values())
        for monitor in monitors :
            monitor.close()













#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import time

class Driver():
    
    def __init__(self,nb_channels=2):
        
        self.nb_channels = int(nb_channels)
        
        for i in range(1,self.nb_channels+1):
            setattr(self,f'channel{i}',Channel(self,i))

        
    def write_to_channel(self,channel,command):
        self.write(f'CHN {channel}')
        self.write(command)

    def idn(self):
        return self.query('*IDN?')
        
        
    def get_driver_model(self):
        model = []
        for num in range(1,self.nb_channels+1) :
            model.append({'element':'module','name':f'channel{num}','object':getattr(self,f'channel{num}')})
        return model

#################################################################################
############################## Connections classes ##############################
class Driver_SOCKET(Driver):
    def __init__(self,address='169.254.62.40',port=9221, **kwargs):
        import socket
        
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((address,int(port)))

        Driver.__init__(self, **kwargs)
    
    def send(self,command):
        self.s.send(command)
    def write(self,query):
        self.s.send((query+'\n').encode())
    def read(self):
        rep = self.s.recv(1000).decode()
        return rep
    def query(self, qry):
        self.write(qry)
        time.sleep(0.2)
        return self.read()
    def close(self):
        self.s.close()
############################## Connections classes ##############################
#################################################################################


class Channel():
    def __init__(self,dev,channel):
        self.channel = int(channel)
        self.dev  = dev
    
    def amplitude(self,amplitude):
        self.dev.write_to_channel(self.channel,f'AMPL {amplitude}')
    def frequency(self,frequency):
        self.dev.write_to_channel(self.channel,f'FREQ {frequency}')
    def offset(self,offset):
        self.dev.write_to_channel(self.channel,f'DCOFFS {offset}')
    
    def set_output(self, state):
        """output on, off, normal or invert""" 
        self.dev.write_to_channel(self.channel,f'OUTPUT {state}')
        
    def load_arb(self, arb_chan):
        """chan: arbitrary channel to load"""
        self.set_mode('ARB')
        self.dev.write_to_channel(self.channel,f'ARBLOAD ARB{arbchan}')
    def write_array_to_byte(self,l,arb_chan):
        """Arguments: array, arbitrary waveform number to address the array to
        Note: ARB1 < BIN > Load data to an existing arbitrary waveform memory location ARB1. The data consists of two bytes per point with no characters between bytes or points. The point data is sent high byte first. The data block has a header which consists of the # character  followed by several ascii coded numeric characters. The first of these defines the number of ascii characters to follow and these following characters define the length of the binary data in bytes. The instrument will wait for data indefinitely If less data is sent. If more data is sent the extra is processed by the command parser which results in a command error."""
        self.set_mode('ARB')
        a = l.astype('<u2').tostring()
        temp = str(2*len(l))
        ARB = str(arb_chan)
        qry = b'ARB'+ bytes(str(ARB),'ascii') +bytes(' #','ascii')+ bytes(str(len(temp)),'ascii')+bytes(temp,'ascii')+a +b' \n'
        self.dev.send(qry)
        time.sleep(0.2)
        
    def set_mode(self,mode_name):
        self.dev.write_to_channel(self.channel,f'WAVE {mode_name}')
        
        
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'amplitude','write':self.amplitude,'type':float,'help':"set the amplitude"})
        model.append({'element':'variable','name':'offset','write':self.offset,'type':float,'help':"Set the offset"})
        model.append({'element':'variable','name':'frequency','write':self.frequency,'type':float,'help':"Set the frequency"})
        model.append({'element':'variable','name':'output','write':self.set_output,'type':str,'help':"Enable/disable the output. Accepted arguments are: ON, OFF"})
        model.append({'element':'variable','name':'set_mode','write':self.set_mode,'type':str,'help':"Set the output mode. Accepted arguments are: SINE, SQUARE, RAMP, TRIANG, PULSE, NOISE, PRBSPN7, PRBSPN9, PRBSPN11, PRBSPN15, PRBSPN20, PRBSPN23, PRBSPN29, PRBSPN31, ARB"})
        return model



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 22 21:03:46 2019

@author: qchat
"""
from threading import Thread, Event
from autolab.core import elements
import collections
import itertools
import os
import time
import h5py

class Scanner :
    
    def __init__(self):
        
        self._parameters = collections.OrderedDict()
        self._recipe = collections.OrderedDict()
        self._initrecipe = collections.OrderedDict()
        self._endrecipe = collections.OrderedDict()
    
        self._name = 'scan'
        self._datapath = os.path.realpath('./')
        
        self._thread = None
        self.verbose = False
        
        
    # Utilities
    # =========================================================================
    
    def _check_item_name(self,name):
        try : 
            assert name not in self._parameters.keys()
            assert name not in self._initrecipe.keys()
            assert name not in self._recipe.keys()
            assert name not in self._endrecipe.keys()
        except :
            raise ValueError(f"Item '{name}' already exists.")
            
    
    def _check_modif_allowed(self):
        assert self._thread is None, f"Scan is running, stop it first to modify it."

            
    def clear(self):
        for obj_name in ['parameters','initrecipe','recipe','endrecipe']:
            self._clear_object(obj_name)

    def _add_item(self,obj_name,item_name,item):
        self._check_modif_allowed()
        self._check_item_name(item_name)
        getattr(self, f'_{obj_name}')[item_name] = item
        
    def _clear_object(self,obj_name):
        self._check_modif_allowed()
        setattr(self, f'_{obj_name}', collections.OrderedDict())
            
        
        
    # Paths
    # =========================================================================
    
    def set_datapath(self,path):
        self._check_modif_allowed()
        path = os.path.realpath(path)
        assert os.path.exists(path), "This path doesn't exists."
        self._datapath = path
        
    def get_datapath(self):
        return self._datapath
    
    
    
    
    # Scan name
    # =========================================================================
    
    def set_name(self,name):
        self._check_modif_allowed()
        self._name = name
        
    def get_name(self,name):
        return self._name


        
    # Parameters and recipe
    # =========================================================================
        
    def add_parameter(self, param_name, parameter):
        assert isinstance(parameter,Parameter), "Wrong object type"
        self._add_item('parameters', param_name, parameter)
        
    def add_recipe_step(self, step_name, step):
        assert isinstance(step,(Measure,Set,Execute,Wait)), "Wrong object type"
        self._add_item('recipe', step_name, step)
    
    def add_init_recipe_step(self, step_name, step):
        assert isinstance(step,(Measure,Set,Execute,Wait)), "Wrong object type"
        self._add_item('initrecipe', step_name, step)
     
    def add_end_recipe_step(self, step_name, step):
        assert isinstance(step,(Measure,Set,Execute,Wait)), "Wrong object type"
        self._add_item('endrecipe', step_name, step)
        
    
    def clear_parameters(self):
        self._clear_object('parameters')

    def clear_recipe(self):
        self._clear_object('recipe')

    def clear_init_recipe(self):
        self._clear_object('intirecipe')

    def clear_end_recipe(self):
        self._clear_object('endrecipe')
    
    
    
    
    
    # Structure
    # =========================================================================
    
    def show_configuration(self):
        
        ''' Prints the current configuration of the scan '''
        
        # Init recipe
        print()
        if len(self._initrecipe) == 0 : print("No init recipe\n"+'='*len('No init recipe'))
        else : 
            print('Init recipe:\n'+'='*len('Init recipe:'))
            i = 1
            for name in self._initrecipe.keys() :
                print(f' {i}) {name}: {self._initrecipe[name].info()}')
                i += 1
                
        # Parameters
        indent = 0
        print()
        if len(self._parameters) == 0 : print("No parameters\n"+'='*len('No parameters'))
        else : 
            print('Parameters:\n'+'='*len('Parameters:'))
            for name in self._parameters.keys() :
                print("   "*indent+f' - {name}: {self._parameters[name].info()}')
                indent += 1
        
        # Recipe
        indent += 1
        print()
        if len(self._recipe) == 0 : print("   "*indent+"No recipe\n"+"   "*indent+'='*len('No recipe'))
        else : 
            print("   "*indent+'Recipe:\n'+"   "*indent+'='*len('Recipe:'))
            i = 1
            for name in self._recipe.keys() :
                print("   "*indent+f' {i}) {name}: {self._recipe[name].info()}')
                i += 1
                
        # End recipe
        print()
        if len(self._endrecipe) == 0 : print("No end recipe\n"+'='*len('No end recipe'))
        else : 
            print('End recipe:\n'+'='*len('End recipe:'))
            i = 1
            for name in self._endrecipe.keys() :
                print(f' {i}) {name}: {self._endrecipe[name].info()}')
                i += 1
        
        
       
        
        
        
    # Scan state
    # =========================================================================
        
    def start(self):
        
        ''' Start a new scan '''
        
        assert self._thread is None, f'The scan is already running'
        self._thread = ScanThread(self)
        self._thread.start()
        if self.verbose : print('Scan started')
        
        
        
    def stop(self):
        
        ''' Stop the ongoing scan '''
        
        assert self._thread is not None, f'The scan is not running.'
        self._thread.stop_event.set()
        self._thread.join()
        self._thread = None
        if self.verbose : print('Scan stopped')
        
        
        
    def pause(self):
        
        ''' Pause the ongoing scan '''
        
        assert self._thread is not None, f'The scan is not running.'
        self._thread.pause_event.set()
        print('Scan paused')
        
        
        
    def resume(self):
        
        ''' Resume the ongoing scan '''
        
        assert self._thread is not None, f'The scan is not running.'
        self._thread.pause_event.clear()
        print('Scan resumed')
        
        
        
        
        
        
        
class ScanThread(Thread):
    
    def __init__(self,scanner):
        
        self.scanner = scanner
        Thread.__init__(self)
        
        # Itertools product on parameters
        self.param_sets = list(dict(zip(self.scanner._parameters.keys(),x)) for x in itertools.product(*[a.values for a in self.scanner._parameters.values()]) )
            
        # Pause and stop events
        self.stop_event = Event()
        self.pause_event = Event()
        
        # Current data
        self.data = collections.OrderedDict()
        
        # Prepare datafile
        self.prepare_datafile()
        
        
        
    def prepare_datafile(self):
        
        ''' Find a unique name for the datafile and initialize it size and name 
        of parameters and step names '''
        
         # Create a unique name for the datafile
        suffix = ''
        count = 0
        while os.path.exists(os.path.join(self.scanner._datapath,self.scanner._name+suffix+'.hdf5')) :
            count += 1
            suffix = f'_{count}'
        self.datapath = os.path.join(self.scanner._datapath,self.scanner._name+suffix+'.hdf5')

        # Prepare structure
        def configure(file,obj,data_length):
            for key in obj.keys(): 
                if isinstance(obj[key],(Parameter,Measure)) :
                    file.create_dataset(key, (data_length,))     
        with h5py.File(self.datapath, "a") as file :
            configure(file,self.scanner._initrecipe,1)
            configure(file,self.scanner._parameters,len(self.param_sets))
            configure(file,self.scanner._recipe,len(self.param_sets))
            configure(file,self.scanner._endrecipe,1)



    def run(self):
        
        ''' Start the execution of the scan '''
        
        # Init recipe
        self.reset_data()
        self.execute_recipe(self.scanner._initrecipe)
        
        # Main recipe of each set of parameter
        self.reset_data()
        for i in range(len(self.param_sets)) :
            self.set_parameters(i)
            self.execute_recipe(self.scanner._recipe,i)
            
        # End recipe
        self.reset_data()
        self.execute_recipe(self.scanner._endrecipe)
        
        
        
    def reset_data(self):
        
        ''' Reset the self.data dictionnary '''
        
        self.data = collections.OrderedDict()
        
        
        
    def execute_recipe(self,recipe,i=0):
        
        """ Execute the given recipe for the i-th time """
        
        for key in recipe.keys() :
            
            # If the scan has not been stopped
            if self.stop_event.is_set() is False :
                
                # Execute step
                step = recipe[key]
                ans = step.execute()
                if ans is not None :
                    self.data[key] = ans
                if self.scanner.verbose : print(key, step.info(), ans)
                
                # If scan is paused, wait for resume
                while self.pause_event.is_set() :
                    time.sleep(0.1)
              
            # If the scan has been stopped
            else :
                break
        
        # Save data
        if self.stop_event.is_set() is False :
            self.save_data(i)
        
        
        
    def set_parameters(self,i):
        
        """ Apply the i-th set of parameters """
        
        param_set = self.param_sets[i]
        
        for key in param_set.keys() :
            
            # If the scan has not been stopped
            if self.stop_event.is_set() is False :
                
                value = param_set[key]
                if key not in self.data.keys() or value != self.data[key] : 
                    parameter = self.scanner._parameters[key]
                    parameter.element(param_set[key])
                    self.data[key] = value
                    if self.scanner.verbose : print(key, parameter.info(), value)
                    
                # If scan is paused, wait for resume
                while self.pause_event.is_set() :
                    time.sleep(0.1)
        
            # If the scan has been stopped
            else :
                break
            
        # Save data
        if self.stop_event.is_set() is False :
            self.save_data(i)
        
        
    
    def save_data(self,i=0):
        
        """ Save in the whole content of the current self.data dictionnary 
        in the hdf5 datafile """
        
        with h5py.File(self.datapath, "a") as file :
            for key in self.data.keys():
                file[key][i] = self.data[key]
        if self.scanner.verbose : print('Saving data')
            
        
        
        
        
        
        
        
        
        
        
        
        
# PARAMETERS AND STEPS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        
class Parameter:
    
    ''' Class dedicated to parameter declaration '''
    
    def __init__(self,element,values):
        assert isinstance(element,elements.Variable), "A parameter must be a Variable"
        assert element.writable is True, f"SET step: Variable {element.address()} is not writable."
        self.element = element
        assert hasattr(values, '__iter__'), "Parameter values must be iterable"
        self.values = values  
        
    def info(self):
        return f"Sweep parameter {self.element.address()} with {len(self.values)} values."
        
        
        
        
class Execute:
    
    ''' Scan step dedicated to execute an Action '''
    
    def __init__(self,element,value=None):
        assert isinstance(element,elements.Action), "EXECUTE step element must be an Action."
        self.element = element
        self.value = None
        if value is not None :
            assert element.has_parameter is True, f"EXECUTE step: element {element.address()} has no parameter."
            try : value = element.type(value)
            except : raise ValueError(f'EXECUTE step: value must be of type {element.type}.')
            self.value = value
    
    def info(self):
        if self.value is None : return f'Execute action {self.element.address()}.'
        else : return f'Execute action {self.element.address()} with value {self.value}.'
        
    def execute(self):
        if self.value is None : self.element()
        else : self.element(self.value)






class Set:
    
    ''' Scan step dedicated to set the value of a Variable '''
    
    def __init__(self,element,value):
        assert isinstance(element,elements.Variable), "SET step element must be a Variable."
        assert element.writable is True, f"SET step: Variable {element.address()} is not writable."
        self.element = element
        try : value = element.type(value)
        except : raise ValueError(f'SET step: value must be of type {element.type}.')
        self.value = value
        
    def info(self):
        return f'Set variable {self.element.address()} with value {self.value}.'
        
    def execute(self):
        self.element(self.value)
        
        
        
        
        
class Measure:
    
    ''' Scan step dedicated to measure a Variable '''

    def __init__(self,element):
        assert isinstance(element,elements.Variable), "MEASURE step element must be a Variable."
        assert element.readable is True, f"MEASURE step: Variable {element.address()} is not readable."
        self.element = element
        
    def info(self):
        return f'Measure variable {self.element.address()}.'
        
    def execute(self):
        return self.element()





class Wait:
    
    ''' Scan step dedicated to pause the scan from a certain amount of time '''
    
    def __init__(self,delay):
        try : delay = float(delay)
        except : raise ValueError('WAIT step delay must be numerical.')
        self.delay = delay
        
    def info(self):
        return f'Wait {self.value} seconds.'
        
    def execute(self):
        time.sleep(self.delay)
 
    
    
# Futures classes, to be defined
 
#class WaitUser:
#    
#    def __init__(self):
#        
#        
#    
#class SendEmail:
#    
#    def __init__(self, address, message):
#        
#        
#       
#class While:
#    
#    def __init__(self, element, '<', value):
#        
#    def execute():
#        while()
#        
        




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import os
import time
from numpy import savetxt,linspace
import pandas

class Driver():
    
    def __init__(self):
        
        for i in ['A','B','C','D','E','F','G']:
            setattr(self,f'trace{i}',Traces(self,i))
            
        
    ### User utilities
    def get_data_traces(self,traces=[],single=None):
        """Get all traces or the ones specified"""
        if single: self.single()
        while not self.is_scope_stopped(): time.sleep(0.05)
        if traces == []: traces = ['A','B','C','D','E','F','G']
        for i in traces:
            #if not(getattr(self,f'trace{i}').is_active()): continue
            getattr(self,f'trace{i}').get_data()
        
    def save_data_traces(self,filename,traces=[],FORCE=False):
        if traces == []: traces = ['A','B','C','D','E','F','G']
        for i in traces:
            getattr(self,f'trace{i}').save_data(filename=filename,FORCE=FORCE)
        
    ### Trigger functions
    def single(self):
        """Trigger a single sweep"""
        self.send('*TRG')
    def is_scope_stopped(self):
        return '1' in self.query(':STATUS:OPER:COND?')


    def get_driver_model(self):
        model = []
        for i in ['A','B','C','D','E','F','G']:
            model.append({'element':'module','name':f'line{i}','object':getattr(self,f'trace{i}'), 'help':'Traces'})
        model.append({'element':'variable','name':'is_stopped','read':self.is_scope_stopped, 'type':bool,'help':'Query whether scope is stopped'})
        model.append({'element':'action','name':'single','do':self.single,'help':'Set single mode'})
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_SOCKET(Driver):
    def __init__(self, address='192.168.0.9', port=10001, **kwargs):
        import socket
        
        self.sock=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((address, int(port)))
        self.send('OPEN "anonymous"')
        ans = self.recv(1024)
        if not ans=='AUTHENTICATE CRAM-MD5.':
            print("problem with authentication")
        self.send(" ")
        ans = self.recv(1024)
        if not ans=='ready':
            print("problem with authentication")
        
        Driver.__init__(self)

    def send(self, msg):
        msg=(msg+"\n").encode()
        self.sock.send(msg)
    def recv(self, length=2048):
        msg=''
        while not msg.endswith('\r\n'):
            msg=msg+self.sock.recv(length).decode()
        msg = msg[:-2]
        return msg
    def query(self,msg,length=2048):
        """Sends question and returns answer"""
        self.send(msg)
        return(self.recv(length))
    def close(self):
        self.sock.close()
############################## Connections classes ##############################
#################################################################################


class Traces():
    def __init__(self,dev,trace):
        self.trace     = str(trace)
        self.dev       = dev
        self.data_dict = {}
        
    def get_data(self):
        self.data        = self.dev.query(f":TRAC:DATA:Y? TR{self.trace}", length=100000).split(',')
        self.frequencies = self.get_frequencies()
        self.data        = [float(val) for val in self.data]
        self.frequencies = [float(val) for val in self.frequencies]
        return self.frequencies,self.data
    def get_data_dataframe(self):
        frequencies,data              = self.get_data()
        self.data_dict['frequencies'] = self.frequencies
        self.data_dict['amplitude']   = self.data
        return pandas.DataFrame(self.data_dict)
    
    def get_frequencies(self):
        return self.dev.query(f":TRAC:DATA:X? TR{self.trace}", length=1000000).split(',')
    
    def save_data(self,filename,FORCE=False):
        temp_filename = f'{filename}_AQ6370TR{self.trace}.txt'
        if os.path.exists(os.path.join(os.getcwd(),temp_filename)) and not(FORCE):
            print('\nFile ', temp_filename, ' already exists, change filename or remove old file\n')
            return
        
        savetxt(temp_filename,(self.frequencies,self.data))


    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'spectrum','read':self.get_data_dataframe,'type':pandas.DataFrame,'help':'Current spectrum'})
        return model
    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Spectrum analyser'
    
class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
        
    autolab driver -d {self.name} -i TCPIP::192.168.0.3::INSTR -l VISA -o my_output_file -c A
    Results in saving one file for the trace A, the data as seen on the scope
    
    autolab driver -d nickname -o my_output_file -c A,B,C
    Same as previous one but with 3 output files on per trace (A, B and C) and using the device nickname as defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the traces to act on/acquire from." )
        parser.add_argument("-o", "--filename", type=str, dest="filename", default=None, help="Set the name of the output file" )
        parser.add_argument("-F", "--force",action="store_true", dest="force", default=None, help="Allows overwriting file" )
        parser.add_argument("-t", "--trigger", dest="trigger",action="store_true", help="Trigger the scope once" )
        
        return parser

    def do_something(self,args):
        if args.trigger and not args.filename:
            getattr(self.Instance,'single')()
        if args.filename:
            getattr(self.Instance,'get_data_traces')(traces=args.channels.split(','),single=args.trigger)
            getattr(self.Instance,'save_data_traces')(filename=args.filename,traces=args.channels.split(','),FORCE=args.force)


    def exit(self):
        self.Instance.close()




# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:29:07 2019

@author: qchat
"""


from PyQt5 import QtCore, QtWidgets
from ..monitoring.main import Monitor
import os
from autolab import paths


class TreeWidgetItemModule(QtWidgets.QTreeWidgetItem):
    
    """ This class represents a module in an item of the tree """
    
    def __init__(self,itemParent,name,gui):
        
        QtWidgets.QTreeWidgetItem.__init__(self,itemParent,[name,'Module'])
        self.setTextAlignment(1,QtCore.Qt.AlignHCenter)
        self.name = name
        self.module = None
        self.loaded = False
        self.gui = gui

        
        
        
    def load(self,module):
        
        """ This function loads the entire module (submodules, variables, actions) """
        
        self.module = module
        
        # Submodules
        subModuleNames = self.module.list_modules()
        for subModuleName in subModuleNames : 
            subModule = getattr(self.module,subModuleName)
            item = TreeWidgetItemModule(self, subModuleName,self.gui)
            item.load(subModule)
            
        # Variables
        varNames = self.module.list_variables()
        for varName in varNames : 
            variable = getattr(self.module,varName)
            TreeWidgetItemVariable(self, variable,self.gui)
            
        
        # Actions
        actNames = self.module.list_actions()
        for actName in actNames : 
            action = getattr(self.module,actName)
            TreeWidgetItemAction(self, action,self.gui)
            
        # Change loaded status
        self.loaded = True
 





       
class TreeWidgetItemAction(QtWidgets.QTreeWidgetItem):
    
    """ This class represents an action in an item of the tree """
    
    def __init__(self,itemParent,action,gui) :
        
        displayName = f'{action.name}'
        if action.unit is not None :
            displayName += f' ({action.unit})'
        
        QtWidgets.QTreeWidgetItem.__init__(self,itemParent,[displayName,'Action'])
        self.setTextAlignment(1,QtCore.Qt.AlignHCenter)
        
        self.gui = gui
        self.action = action     
        
        if self.action.has_parameter :
            if self.action.type in [int,float,str] : 
                self.executable = True
                self.has_value = True
            else : 
                self.executable = False
                self.has_value = False
        else : 
            self.executable = True
            self.has_value = False
            
        # Main - Column 2 : actionread button
        if self.executable is True : 
            self.execButton = QtWidgets.QPushButton()
            self.execButton.setText("Execute")
            self.execButton.pressed.connect(self.execute)
            self.gui.tree.setItemWidget(self, 2, self.execButton)
        
        # Main - Column 3 : QlineEdit if the action has a parameter
        if self.has_value :
            self.valueWidget = QtWidgets.QLineEdit()
            self.valueWidget.setAlignment(QtCore.Qt.AlignCenter)
            self.gui.tree.setItemWidget(self, 3, self.valueWidget)
               
        # Tooltip
        if self.action._help is None : tooltip = 'No help available for this action'
        else : tooltip = self.action._help
        self.setToolTip(0,tooltip)
        
        
    def readGui(self):
        
        """ This function returns the value in good format of the value in the GUI """
        
        value = self.valueWidget.text()
        if value == '' :
            self.gui.statusbar.showMessage(f"Action {self.action.name} requires a value for its parameter",10000)
        else : 
            try : 
                value = self.action.type(value)
                return value
            except :
                self.gui.statusbar.showMessage(f"Action {self.action.name}: Impossible to convert {value} in type {self.action.type.__name__}",10000)
            
        
        
    def execute(self):
        
        """ Start a new thread to execute the associated action """
        
        if self.has_value :
            value = self.readGui()
            if value is not None : self.gui.threadManager.start(self,'execute',value=value)
        else : 
            self.gui.threadManager.start(self,'execute')
                


    def menu(self,position):
        
        """ This function provides the menu when the user right click on an item """
        
        menu = QtWidgets.QMenu()            
        scanRecipe = menu.addAction("Do in scan recipe")
            
        choice = menu.exec_(self.gui.tree.viewport().mapToGlobal(position))
            
        if choice == scanRecipe : 
            self.gui.addStepToScanRecipe('action',self.action)


        
        
        
        
class TreeWidgetItemVariable(QtWidgets.QTreeWidgetItem):   

    """ This class represents a variable in an item of the tree """     
    
    def __init__(self,itemParent,variable,gui) :
        
        displayName = f'{variable.name}'
        if variable.unit is not None :
            displayName += f' ({variable.unit})'

        QtWidgets.QTreeWidgetItem.__init__(self,itemParent,[displayName,'Variable'])
        self.setTextAlignment(1,QtCore.Qt.AlignHCenter)
        
        self.gui = gui

        self.variable = variable
        
        self.monitor = None
        
        # Signal creation and associations in autolab devices instances         
        self.readSignal = ReadSignal()
        self.readSignal.signal.connect(self.writeGui)
        self.variable._read_signal = self.readSignal
        self.writeSignal = WriteSignal()
        self.writeSignal.signal.connect(self.valueEdited)
        self.variable._write_signal = self.writeSignal
        
        # Main - Column 2 : Creation of a READ button
        if self.variable.readable and self.variable.type in [int,float,bool,str] :
            self.readButton = QtWidgets.QPushButton()
            self.readButton.setText("Read")
            self.readButton.pressed.connect(self.read)
            self.gui.tree.setItemWidget(self, 2, self.readButton)
        
        # Main - column 3 : Creation of a VALUE widget, depending on the type, and if the variable is readable
        
        ## QLineEdit or QLabel 
        if self.variable.type in [int,float,str] : 
            
            if self.variable.writable :
                self.valueWidget = QtWidgets.QLineEdit()
                self.valueWidget.setAlignment(QtCore.Qt.AlignCenter)
                self.valueWidget.returnPressed.connect(self.write)
                self.valueWidget.textEdited.connect(self.valueEdited)
            else :
                self.valueWidget = QtWidgets.QLabel()
                self.valueWidget.setAlignment(QtCore.Qt.AlignCenter)
            self.gui.tree.setItemWidget(self, 3, self.valueWidget)
            
        ## QCheckbox for boolean variables
        elif self.variable.type in [bool] :
            self.valueWidget = QtWidgets.QCheckBox()
            self.valueWidget.stateChanged.connect(self.valueEdited)
            self.valueWidget.stateChanged.connect(self.write)
            hbox = QtWidgets.QHBoxLayout()
            hbox.addWidget(self.valueWidget)
            hbox.setAlignment(QtCore.Qt.AlignCenter)
            hbox.setSpacing(0)
            hbox.setContentsMargins(0,0,0,0)
            widget = QtWidgets.QWidget()
            widget.setLayout(hbox)
            if self.variable.writable is False : # Disable interaction is not writable
                self.valueWidget.setEnabled(False)
            self.gui.tree.setItemWidget(self, 3, widget)
            
        # Main - column 4 : indicator (status of the actual value : known or not known)
        if self.variable.type in [int,float,str,bool] :
            self.indicator = QtWidgets.QLabel()
            self.gui.tree.setItemWidget(self, 4, self.indicator)
            
        # Tooltip
        if self.variable._help is None : tooltip = 'No help available for this variable'
        else : tooltip = self.variable._help
        self.setToolTip(0,tooltip)
                   
            
    def writeGui(self,value):
        
        """ This function displays a new value in the GUI """
        
        # Update value
        if self.variable.numerical :
            self.valueWidget.setText(f'{value:g}')
        elif self.variable.type in [str] :
            self.valueWidget.setText(value)
        elif self.variable.type in [bool] :
            self.valueWidget.setChecked(value)
            
        # Change indicator light to green
        if self.variable.type in [int,float,bool,str] :
            self.setValueKnownState(True)
            
            
            
    def readGui(self):
        
        """ This function returns the value in good format of the value in the GUI """
        
        if self.variable.type in [int,float,str] :
            value = self.valueWidget.text()
            if value == '' :
                self.gui.statusbar.showMessage(f"Variable {self.variable.name} requires a value to be set",10000)
            else :
                try : 
                    value = self.variable.type(value)
                    return value
                except : 
                    self.gui.statusbar.showMessage(f"Variable {self.variable.name}: Impossible to convert {value} in type {self.variable.type.__name__}",10000)
        
        elif self.variable.type in [bool] :
            value = self.valueWidget.isChecked()
            return value


    def setValueKnownState(self,state):
        
        """ Turn the color of the indicator depending of the known state of the value """
        
        if state is True : self.indicator.setStyleSheet(f"background-color:#70db70") #green
        else : self.indicator.setStyleSheet(f"background-color:#ff8c1a") #orange
            
            
    
    def read(self):
        
        """ Start a new thread to READ the associated variable """
        
        self.setValueKnownState(False)
        self.gui.threadManager.start(self,'read')
        
        
        
    def write(self):
        
        """ Start a new thread to WRITE the associated variable """
        value = self.readGui()
        if value is not None :
            self.gui.threadManager.start(self,'write',value=value)


        
        
    def valueEdited(self):
        
        """ Function call when the value displayed in not sure anymore.
            The value has been modified either in the GUI (but not sent) or by command line """
            
        self.setValueKnownState(False)
        
        
        
    def menu(self,position):
        
        """ This function provides the menu when the user right click on an item """
                
        menu = QtWidgets.QMenu()
        
        monitoringAction = menu.addAction("Start monitoring")
        menu.addSeparator()
        scanParameterAction = menu.addAction("Set as scan parameter")
        scanMeasureStepAction = menu.addAction("Measure in scan recipe")
        scanSetStepAction = menu.addAction("Set value in scan recipe")
        menu.addSeparator()
        saveAction = menu.addAction("Read and save as...")
        
        monitoringAction.setEnabled( self.variable.readable and self.variable.numerical )
        scanParameterAction.setEnabled(self.variable.parameter_allowed)
        scanMeasureStepAction.setEnabled(self.variable.readable)
        saveAction.setEnabled(self.variable.readable)
        scanSetStepAction.setEnabled(self.variable.writable)
        
                        
        choice = menu.exec_(self.gui.tree.viewport().mapToGlobal(position))

        if choice == monitoringAction : 
            self.openMonitor()
        elif choice == scanParameterAction :
            self.gui.setScanParameter(self.variable) 
        elif choice == scanMeasureStepAction :
            self.gui.addStepToScanRecipe('measure',self.variable) 
        elif choice == scanSetStepAction :
            self.gui.addStepToScanRecipe('set',self.variable) 
        elif choice == saveAction :
            self.saveValue()
            
    def saveValue(self):
        
        path = QtWidgets.QFileDialog.getSaveFileName(self.gui, f"Save {self.variable.name} value", 
                                        os.path.join(paths.USER_LAST_CUSTOM_FOLDER,f'{self.variable.address()}.txt'), 
                                        "Text file (*.txt)")[0]
        if path != '' :
            paths.USER_LAST_CUSTOM_FOLDER = path
            try : 
                self.gui.statusbar.showMessage(f"Saving value of {self.variable.name}...",5000)
                self.variable.save(path)
                self.gui.statusbar.showMessage(f"Value of {self.variable.name} successfully read and save at {path}",5000)
            except Exception as e :
                self.gui.statusbar.showMessage(f"An error occured: {str(e)}",10000)
            


    def openMonitor(self):
        
        """ This function open the monitor associated to this variable. """
        
        # If the monitor is not already running, create one
        if id(self) not in self.gui.monitors.keys(): 
            self.gui.monitors[id(self)] = Monitor(self)
            self.gui.monitors[id(self)].show()
        
        # If the monitor is already running, just make as the front window
        else :  
            self.gui.monitors[id(self)].setWindowState(self.monitor.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
            self.gui.monitors[id(self)].activateWindow()
        

        
        
    def clearMonitor(self):
        
        """ This clear the gui instance reference when quitted """
        if id(self) in self.gui.monitors.keys(): 
            self.gui.monitors.pop(id(self))
            
            
            
# Signals can be emitted only from QObjects  
# These class provides convenient ways to use signals
class ReadSignal(QtCore.QObject):
    signal = QtCore.pyqtSignal(object)
    def emit(self,value):
        self.signal.emit(value)
        
class WriteSignal(QtCore.QObject):
    signal = QtCore.pyqtSignal()
    def emit(self):
        self.signal.emit()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
WARNING: The message processing to submodules is not optimal as connecting/disconnecting streams for each commands: should be optimized. See methods send_command_to_slot and get_query_from_slot of the Driver class.

Supported instruments (identified):
- 

"""

import time


class Driver():

    slot_config = '<MODULE_NAME>'
    
    def __init__(self, **kwargs):
        
        self.write('CEOI ON')
        self.write('EOIX ON')
        self.write('TERM LF')
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name = kwargs[key].strip()
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,slot_num))
                self.slot_names[slot_num] = name
                
        
    
    def send_command_to_slot(self,slot,command):
        self.write(f'CONN {slot},"CONAME"')
        self.write(command)
        self.write("CONAME")
    def get_query_from_slot(self,slot,command):
        self.write(f'CONN {slot},"CONAME"')
        rep = self.query(command)
        self.write("CONAME")
        return rep

    def idn(self):
        return self.query('*IDN?')


    def get_driver_model(self):
        config = []
        for name in self.slot_names.values() :
            config.append({'element':'module','name':name,'object':getattr(self,name)})
        return config

#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        import visa
        
        rm = visa.ResourceManager()
        self.inst = rm.get_instrument(address)
        
        Driver.__init__(self, **kwargs)
        
    def close(self):
        self.inst.close()
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self):
        rep = self.inst.read()
        return rep
    
class Driver_GPIB(Driver):
    def __init__(self,address=2,board_index=0,**kwargs):
        import Gpib
        
        self.inst = Gpib.Gpib(int(board_index),int(address))
        Driver.__init__(self, **kwargs)
    
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,command):
        self.inst.write(command)
    def read(self,length=1000000000):
        return self.inst.read(length).decode().strip('\r\n')
    def close(self):
        """WARNING: GPIB closing is automatic at sys.exit() doing it twice results in a gpib error"""
        #Gpib.gpib.close(self.inst.id)
        pass
############################## Connections classes ##############################
#################################################################################


class Module_SIM960():
    
    category = 'PID controller'
    
    def __init__(self,dev,slot):
        self.slot = int(slot)
        self.dev  = dev
        

    def set_output_manual(self):
        self.dev.send_command_to_slot(self.slot,'AMAN 0')
    def set_output_pid(self):
        self.dev.send_command_to_slot(self.slot,'AMAN 1')
    def set_output_manual_voltage(self,val):
        self.dev.send_command_to_slot(self.slot,f'MOUT {val}')
    def get_output_voltage(self):
        return float(self.dev.get_query_from_slot(self.slot,'OMON?'))
    def set_setpoint(self,val):
        self.dev.send_command_to_slot(self.slot,f'SETP {setpoint}')

    def auto_lock(self,peculiar=False):
        rep = self.get_output_voltage()
        if peculiar:                     # port 5
            if rep<1.5 or rep>8.5:
                self.re_lock()
        else:
            if rep<-3.5 or rep>3.5:
                self.re_lock(port)
                
    def relock(self):
        self.set_output_manual()
        time.sleep(0.1)
        if peculiar:                     # port 5
            self.set_output_manual_voltage(5)
        else:
            self.set_output_manual_voltage(0)
        time.sleep(0.1)
        self.set_output_pid()


    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'setpoint','type':float,'write':self.set_setpoint,'help':'Setpoint value to be used in Volts'})
        model.append({'element':'variable','name':'output_voltage','type':float,'read':self.get_output_voltage,'help':'Get the ouptut voltage'})
        model.append({'element':'variable','name':'output_manual_voltage','type':float,'write':self.set_output_manual_voltage,'help':'Set the manual output voltage value'})
        model.append({'element':'action','name':'PID_on','do':self.set_output_pid,'help':'Set output mode to PID'})
        model.append({'element':'action','name':'PID_off','do':self.set_output_manual,'help':'Set output mode to manual'})
        model.append({'element':'action','name':'relock','do':self.relock,'help':'Try to relock'})
        model.append({'element':'action','name':'auto_lock','do':self.auto_lock,'help':'Choose automatically to unlock and relock in order to decrease the output voltage'})
        return model





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

from XPS import XPS


class Driver():

    slot_config = '<MODULE_NAME>,<NAME_IN_XPS>,<CALIBRATION_PATH>'

    def __init__(self,**kwargs):
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name, name_in_xps, calib_path = [a.strip() for a in kwargs[key].split(',')]
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,name_in_xps,calib_path))
                self.slot_names[slot_num] = name

        
    def get_driver_model(self):
        model = [{'element':'module','name':name,'object':getattr(self,name)}
                 for name in self.slot_names.values() ]
        return model

    
    
#################################################################################
############################## Connections classes ##############################
class Driver_SOCKET(Driver):
    
    def __init__(self,address='192.168.0.8',**kwargs):
    
        self.TIMEOUT = 2
        
        # Instantiation
        self.controller = XPS()
        self.socketID = self.controller.TCP_ConnectToServer(address,5001,self.TIMEOUT)

        Driver.__init__(self,**kwargs)


    def isConnected(self):
        try :
            assert self.controller.FirmwareVersionGet(self.socketID) is not None
            return True
        except :
            return False
        
        
    def query(self,command,**kwargs):
        
        assert isinstance(command,list)
        assert hasattr(self.controller,command[0])

        if len(command)>1 :
            ans = getattr(self.controller,command[0])(self.socketID,*command[1:])
        else :
            ans = getattr(self.controller,command[0])(self.socketID)
            
        if ans[1] != '' :
            if len(ans)>2:
                return ans[1:]
            else :
                return ans[1]
        
    def close(self):
        try : self.controller.TCP_CloseSocket(self.socketId)
        except : pass
         
############################## Connections classes ##############################
#################################################################################
    
import time
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import savgol_filter
from scipy.interpolate import interp1d

class Module_NSR1():
    
    category = 'Rotation stage'
    
    def __init__(self,dev,slot,calibration_path):
        
        self.dev = dev
        self.NAME = slot
        self.SLOT = slot
        
        self.calibration_path = calibration_path
        assert os.path.exists(calibration_path)
        self.calibration_function = None
        
        self.calib = None
        self.load_calibration()
        
    #--------------------------------------------------------------------------
    # Calibration functions
    #--------------------------------------------------------------------------
    
    def load_calibration(self):
        try: self.calib = pd.read_csv(os.path.join(self.calibration_path,'calib.csv'))
        except: pass
    
    def set_calibration_function(self,calibration_function):
        self.calibration_function = calibration_function
        
    def calibrate(self):
        
        assert self.calibration_function is not None
        
        def scan(list_angle):
            
            df = pd.DataFrame()

            for angle_setpoint in list_angle :
                self.set_angle(angle_setpoint)
                angle=self.get_angle()
                power=self.calibration_function()
                df=df.append({'angle':angle,'power':power})
                   
            df.sort_values(by=['angle'],inplace=True)
            return df
        
        plt.ioff()
        fig = plt.figure()
        ax=fig.add_subplot(111)
        
        # Homing
        self.go_home()
        
        # Fabrication de la liste des angles à explorer
        list_angle = np.concatenate((np.arange(0,360,8),np.arange(4,360,8)))
        list_angle = np.flipud(list_angle)
        
        # Lancement du scan de mesure de puissance
        df=scan(list_angle)
        
        # Find transition
        derivative = np.diff(df.power)
        imax=np.argmax(derivative)
        
        # Recoupage
        df_shifted = pd.DataFrame()
        df_shifted['power'] = np.concatenate((df.power[imax:],df.power[0:imax]))
        df_shifted['angle'] = np.concatenate((df.angle[imax:]-360,df.angle[0:imax]))
        df = df_shifted
        
        # Suppression transition sur 10deg
        df = pd.DataFrame()
        df = df[df.angle>(df.angle.min()+10)]
        df = df[df.angle<(df.angle.max()-10)]
        
        # Normalisation
        df['transmission']=df.power/df.power.max()
        ax.plot(df.angle,df.transmission,'x',label='raw')
        
        # Lissage
        df['transmission']=savgol_filter(df.transmission, 7, 3)
        
        # Interpolation
        interpolFunc=interp1d(df.angle, df.transmission, kind='cubic')
        df['angle']=np.linspace(df.angle.min(),df.angle.max(),1000)
        df['transmission']=interpolFunc(df.angle)
        
        # EXTRACT On ne garde que entre 0% et 95% car trop bruité au dessus
        df = df[df.transmission<0.95]
        
        # Sauvegarde plot
        date=time.strftime("%Y%m%d_%H%M%S")
        ax.plot(df.angle,df.transmission,'r-',label='Final')
        ax.set_title(f'{self.NAME} filter {date}')
        ax.grid()
        ax.set_xlabel('Angle [deg]')
        ax.set_ylabel('Power [a.u.]')
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(handles, labels, loc=0)
        filepath=os.path.join(self.calibration_path,f'{date}_{self.NAME}_calib.jpg')
        fig.savefig(filepath,bbox_inches='tight',dpi=300)
        fig.close()
           
        # Enregistrement des données 
        df.to_csv(os.path.join(self.calibration_path,'calib.csv'))
        
        # Raffraichissement des donnees
        self.load_calibration()
        

        
        
        
        
    #--------------------------------------------------------------------------
    # Optional functions
    #--------------------------------------------------------------------------
        
    def safe_state(self):
        self.set_min()
        if self.get_transmission() < 0.2 :
            return True
            

        

        
    #--------------------------------------------------------------------------
    # Technical functions
    #--------------------------------------------------------------------------
    
    def get_filter_state(self):
        state=self.dev.query(['GroupStatusGet',self.SLOT])
        
        if 0 <= state <= 9 :
            return 'NOTINIT'
        elif state == 42 :
            return 'NOTREF'
        elif state == 43 :
            return 'HOMING'
        elif state == 44 :
            return 'MOVING'
        elif 10 <= state <= 19 :
            return 'ENABLED'
        elif 20 <= state <= 39 :
            return 'DISABLED'
        else :
            return None
    
    def check_ready_state(self):
    
        if self.get_filter_state() in ['ENABLED','DISABLED'] :
            return True 
        else :
            if self.get_filter_state() == 'NOTINIT' :
                self.dev.query(['GroupInitialize',self.SLOT])
                if self.get_filter_state() != 'NOTREF' :
                    return False
            if self.get_filter_state() == 'NOTREF':
                self.dev.query(['GroupHomeSearch',self.SLOT])
                while self.get_filter_state() == 'HOMING' :
                    time.sleep(0.5)
                if self.get_filter_state() in ['ENABLED','DISABLED'] :
                    self.set_enabled(False)
                    return True
                else :
                    return False
                
    
    def wait_move_ending(self):
        while self.get_filter_state() == 'MOVING' :
            time.sleep(0.1)
        time.sleep(0.4) # Trop rapide sinon
        
        
        
        
        
        
    def set_enabled(self,state):
        assert isinstance(state,bool)
        if state is True :
            self.dev.query(['GroupMotionEnable',self.SLOT])
        else :
            self.dev.query(['GroupMotionDisable',self.SLOT])       

    def is_enabled(self):
        state=self.get_filter_state()
        if state in ['ENABLED','MOVING'] :
            return True
        else :
            return False
        
        
        
        

    def set_positioner_name(self,value):
        assert isinstance(value,str)
        self.set_data('positionerName',value)
        
    def get_positioner_name(self):
        if 'positionerName' not in self.get_dataList() :
            self.set_positioner_name('Pos')
        return self.get_data('positionerName')
    
    
    
        
    #--------------------------------------------------------------------------
    # Instrument variables
    #--------------------------------------------------------------------------
    
    
    
    def get_parameters(self):
        if self.check_ready_state() is True :
            params={}
            temp=self.dev.query(['PositionerSGammaParametersGet',self.SLOT+'.'+self.get_positioner_name()])       
            params['velocity']=temp[0]
            params['acceleration']=temp[1]
            params['minJerkTime']=temp[2]
            params['maxJerkTime']=temp[3]
            return params
        else :
            raise ValueError('Not ready')
            
    def set_parameters(self,params):
        if self.check_ready_state() is True :
            self.dev.query(['PositionerSGammaParametersSet',self.SLOT+'.'+self.get_positioner_name(),
                                                             params['velocity'],
                                                             params['acceleration'],
                                                             params['minJerkTime'],
                                                             params['maxJerkTime']]) 
        else :
            raise ValueError('Not ready')






    def set_velocity(self,value):
        assert isinstance(float(value),float)
        value=float(value)
        params=self.get_parameters()
        params['velocity']=value
        self.set_parameters(params)
    
    def get_velocity(self):
        return float(self.get_parameters()['velocity'])


        
    


    def set_acceleration(self,value):
        assert isinstance(float(value),float)
        value=float(value)
        params=self.get_parameters()
        params['acceleration']=value
        self.set_parameters(params)
    
    def get_acceleration(self):
        return float(self.get_parameters()['acceleration'])
    



    
    
    def set_angle(self,value,forced=False):
        assert isinstance(float(value),float)
        value=float(value)
        
        if forced is False :
            curr_angle = self.get_angle()
            if value > curr_angle - 1.9 :
                self.set_angle(value+20,forced=True)
            
        if self.check_ready_state() is True :
            self.set_enabled(True)
            self.dev.query(['GroupMoveAbsolute',self.SLOT,[value]])
            self.wait_move_ending()
            self.set_enabled(False)
        else :
            raise ValueError('Not ready')
        
    def get_angle(self):
        if self.check_ready_state() is True :
            value = float(self.dev.query(['GroupPositionCurrentGet',self.SLOT,1]))
            return value
        else :
            raise ValueError('Not ready')
    
    
    
    
    

    def set_transmission(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        angle = self.get_angle_from_transmission(value)
        self.set_angle(angle)
    
    def get_transmission(self):
        angle = self.get_angle()
        return self.get_transmission_from_angle(angle)

        
    def get_transmission_max(self):
        return self.calib.transmission.max()



    def get_angle_from_transmission(self,value):
        ind = abs(self.calib.transmission-value).idxmin()
        return float(self.calib.loc[ind,'angle'])   
    
    def get_transmission_from_angle(self,value):
        ind = abs(self.calib.angle-value).idxmin()
        return float(self.calib.loc[ind,'transmission'])   





        



    
      
    
    def set_min(self):
        self.set_transmission(0)
        
    def set_max(self):
        self.set_transmission(1)



            
    
    def go_home(self):
        
        self.set_angle(self.get_angle()+5) # In case we are at home - blocking
        
        # Go to not init
        self.dev.query(['GroupKill',self.SLOT])
        while self.get_filter_state() != 'NOTINIT':
            time.sleep(0.1)
                            
        # Go from not init to ref mode
        self.dev.query(['GroupInitialize',self.SLOT])
        while self.get_filter_state() != 'NOTREF':
            time.sleep(0.1)
        
        # Homing to get back to ready state
        self.dev.query(['GroupHomeSearch',self.SLOT])
        while self.get_filter_state() != 'ENABLED':
            time.sleep(0.1)

        # On le désactive
        self.set_enabled(False)
        while self.get_filter_state() != 'DISABLED':
            time.sleep(0.1)
            
            
            
            
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'velocity','type':float,'read':self.get_velocity,'write':self.set_velocity,'help':'Velocity of the filter during move'})
        model.append({'element':'variable','name':'acceleration','type':float,'read':self.get_acceleration,'write':self.set_acceleration,'help':'Acceleration of the filter during move'})
        model.append({'element':'variable','name':'angle','type':float,'read':self.get_angle,'write':self.set_angle,'help':'Current angle position'})
        model.append({'element':'variable','name':'transmission','type':float,'read':self.get_transmission,'write':self.set_transmission,'help':'Current transmission of the filter'})
        model.append({'element':'action','name':'set_min','do':self.set_min,'help':'Go to minimum transmission'})
        model.append({'element':'action','name':'set_max','do':self.set_max,'help':'Go to maximum transmission'})
        model.append({'element':'action','name':'go_home','do':self.go_home,'help':'Go to home position'})
        return model
    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- Wavemaster 8620A
- Waverunner 104Xi
- Waverunner 6050A
"""

import os
import time
from numpy import int8,int16,frombuffer
import numpy as np


class Driver():
    
    def __init__(self,nb_channels=4):
        
        self.nb_channels = int(nb_channels)
        self.encoding    = 'BYTE'
        
        self.write('CFMT DEF9,'+self.encoding+',BIN')
        self.write('CHDR SHORT')
        
        for i in range(1,self.nb_channels+1):
            setattr(self,f'channel{i}',Channel(self,i))
    
    
    ### User utilities
    def get_data_channels(self,channels=[],single=False):
        """Get all channels or the ones specified"""
        previous_trigger_state = self.get_previous_trigger_state()
        self.stop()
        if single: self.single()
        while not self.is_stopped(): time.sleep(0.05)
        if channels == []: channels = list(range(1,self.nb_channels+1))
        for i in channels:
            if not(getattr(self,f'channel{i}').is_active()): continue
            getattr(self,f'channel{i}').get_data_raw()
            getattr(self,f'channel{i}').get_log_data()
        self.set_previous_trigger_state(previous_trigger_state)
        
    def save_data_channels(self,filename,channels=[],FORCE=False):
        if channels == []: channels = list(range(1,self.nb_channels+1))
        for i in channels:
            getattr(self,f'channel{i}').save_data_raw(filename=filename,FORCE=FORCE)
            getattr(self,f'channel{i}').save_log_data(filename=filename,FORCE=FORCE)
        
    ### Trigger functions
    def single(self):
        self.write("TRMD SINGLE")
    def stop(self):
        self.write("TRMD STOP")
    def auto(self):
        self.write("TRMD AUTO")
    def normal(self):
        self.write("TRMD NORMAL")
    def is_stopped(self):
        return 'STOP' in self.query('TRMD?')
    def get_previous_trigger_state(self):
        return self.query('TRMD?')
        
    def set_previous_trigger_state(self,previous_trigger_state):
        self.write(previous_trigger_state)
        
    ### Cross-channel settings 
    def set_encoding(self,encoding):
        self.encoding = encoding
        self.write('CFMT DEF9,'+self.encoding+',BIN')
    def get_encoding(self):
        return self.encoding
  
    def get_driver_model(self):
        
        model = []
        for num in range(1,self.nb_channels+1) :
            model.append({'element':'module','name':f'channel{num}','object':getattr(self,f'channel{num}')})
        
        model.append({'element':'variable','name':'is_stopped','read':self.is_stopped, 'type':bool,'help':'Query whether scope is stopped'})
        model.append({'element':'variable','name':'encoding','write':self.set_encoding,'read':self.get_encoding, 'type':str,'help':'Set the data encoding too use. Accepted values are: BYTE, WORD, ... Default value is BYTE'})
        model.append({'element':'action','name':'single','do':self.single,'help':'Set single mode for trigger'})
        model.append({'element':'action','name':'stop','do':self.stop,'help':'Set stop mode for trigger'})
        model.append({'element':'action','name':'auto','do':self.auto,'help':'Set auto mode for trigger'})
        model.append({'element':'action','name':'normal','do':self.normal,'help':'Set normal mode for trigger'})
        return model
    
#################################################################################
############################## Connections classes ##############################
#class Driver_VISA(Driver):
    #def __init__(self, address='GPIB0::2::INSTR', **kwargs):
        #import visa as v
        
        #rm        = v.ResourceManager()
        #self.inst = rm.get_instrument(address)
        #Driver.__init__(self, **kwargs)
    
    #def query(self,command):
        #self.write(command)
        #return self.read()
    #def read(self):
        #return self.inst.read()
    #def read_raw(self):
        #return self.inst.read_raw()
    #def write(self,command):
        #self.inst.write(command)
    #def close(self):
        #self.inst.close()

class Driver_VXI11(Driver):
    def __init__(self, address='192.168.0.1', **kwargs):
        import vxi11 as v
    
        self.inst = v.Instrument(address)
        Driver.__init__(self, **kwargs)

    def query(self, command, nbytes=100000000):
        self.write(command)
        return self.read(nbytes)
    def read(self,nbytes=100000000):
        return self.inst.read(nbytes)
    def read_raw(self):
        return self.inst.read_raw()
    def write(self,cmd):
        self.inst.write(cmd)
    def close(self):
        self.inst.close()
############################## Connections classes ##############################
#################################################################################


class Channel():
    def __init__(self,dev,channel):
        self.channel          = int(channel)
        self.dev              = dev
        self.autoscale        = False
        self.autoscale_factor = 8
    
    
    def get_data_raw(self):
        if self.autoscale:
            self.do_autoscale()
        self.dev.write(f'C{self.channel}:WF? DAT1')
        self.data_raw = self.dev.read_raw()
        self.data_raw = self.data_raw[self.data_raw.find(b'#')+11:-1]
        return self.data_raw
    def get_data(self):
        if self.dev.encoding=='BYTE': return frombuffer(self.get_data_raw(),int8)
        if self.dev.encoding=='WORD': return frombuffer(self.get_data_raw(),int16)
    def get_log_data(self):
        self.log_data = self.dev.query(f"C{self.channel}:INSP? 'WAVEDESC'")
        return self.log_data
    
    
    def save_data_raw(self,filename,FORCE=False):
        temp_filename = f'{filename}_WAVEMASTERCH{self.channel}'
        if os.path.exists(os.path.join(os.getcwd(),temp_filename)) and not(FORCE):
            print('\nFile ', temp_filename, ' already exists, change filename or remove old file\n')
            return
        f = open(temp_filename,'wb')# Save data
        f.write(self.data_raw)
        f.close()
    def save_log_data(self,filename,FORCE=False):
        temp_filename = f'{filename}_WAVEMASTERCH{self.channel}.log'
        if os.path.exists(os.path.join(os.getcwd(),temp_filename)) and not(FORCE):
            print('\nFile ', temp_filename, ' already exists, change filename or remove old file\n')
            return
        f = open(temp_filename,'w')
        f.write(self.log_data)
        f.close()
    
    def get_data_numerical(self):
        return array_of_float
    def save_data_numerical(self):
        return array_of_float
    
    # additionnal functions
    def get_min(self):
        stri  = self.dev.query(f'C{self.channel}:PAVA? MIN')
        return float(stri.split(',')[1].split(' ')[0])
    def get_max(self):
        stri  = self.dev.query(f'C{self.channel}:PAVA? MAX')
        return float(stri.split(',')[1].split(' ')[0])
    def get_mean(self):
        stri = self.dev.query(f'C{self.channel}:PAVA? MEAN')
        return float(stri.split(',')[1].split(' ')[0])
        
    def set_vertical_offset(self,val):
        self.dev.write(f'C{self.channel}:OFST {val}') 
    def get_vertical_offset(self):
        return float(self.dev.query(f'C{self.channel}:OFST?').split(' ')[1])
    def set_vertical_scale(self,val):
        self.dev.write(f'C{self.channel}:VDIV {val}') 
    def get_vertical_scale(self):
        return float(self.dev.query(f'C{self.channel}:VDIV?').split(' ')[1])
        
    def do_autoscale(self):
        fact       = 0
        target     = self.autoscale_factor * 0.1
        acceptance = 0.05
        
        mi = self.get_min()
        ma = self.get_max()
        val = round((-1)*(mi+ma)/2.,3)
        self.set_vertical_offset(val)
        VDIV = self.get_vertical_scale()
        OFST = self.get_vertical_offset()
        R_MI = -(5*VDIV)-OFST
        R_MA =   5*VDIV -OFST
        r_diff = R_MA - R_MI
        fact = (ma-mi)/r_diff
        
        k = 1
        while abs(target-fact)>acceptance or mi<R_MI or ma>R_MA or abs(R_MI-mi)<0.10*VDIV*10:            
            # compute new channel amplitude as distance from target
            new_channel_amp  = VDIV + VDIV * round(fact-target,3)
            if mi<R_MI or ma>R_MA or abs(R_MI-mi)<0.10*VDIV*10:
                new_channel_amp = new_channel_amp + new_channel_amp * 0.5                    
            if new_channel_amp<0.005: new_channel_amp = 0.005 # if lower than the lowest possible 5mV/div
            self.set_vertical_scale(new_channel_amp)  # set_vertical_scale do change vertical_offset...
            self.set_vertical_offset(val)             # must do this then
            
            # trig for a eventual new iteration
            self.dev.single()
            print(f'Optimisation loop, index: {k}, distance from target ({acceptance}): {round(abs(target-fact),3)}')
            k = k + 1
            
            mi = self.get_min()
            ma = self.get_max()
            val = round((-1)*(mi+ma)/2.,3)
            self.set_vertical_offset(val)
            VDIV = self.get_vertical_scale()
            OFST = self.get_vertical_offset()
            R_MI = -(5*VDIV)-OFST
            R_MA =   5*VDIV -OFST
            r_diff = R_MA - R_MI
            fact = (ma-mi)/r_diff
            
            
    def set_autoscale_enable(self):
        self.autoscale = True
    def set_autoscale_disable(self):
        self.autoscale = False
    def set_autoscale_factor(self,val):
        self.autoscale_factor = float(val)
    def get_autoscale_factor(self):
        return self.autoscale_factor
           
    def is_active(self):
        temp = self.dev.query(f'C{self.channel}:TRA?')
        return 'ON' in temp



    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'min','type':float,'unit':'V','read':self.get_min,'help':'Get minimum value of the current signal'})
        model.append({'element':'variable','name':'max','type':float,'unit':'V','read':self.get_max,'help':'Get maximum value of the current signal'})
        model.append({'element':'variable','name':'mean','type':float,'unit':'V','read':self.get_mean,'help':'Get mean value of the current signal'})
        model.append({'element':'variable','name':'trace_raw','type':bytes,'read':self.get_data_raw,'help':'Get the current trace in bytes'})
        model.append({'element':'variable','name':'trace','type':np.ndarray,'read':self.get_data,'help':'Get the current trace in numpy'})
        model.append({'element':'variable','name':'vertical_scale','type':float,'unit':'V/div','read':self.get_vertical_scale,'write':self.set_vertical_scale,'help':'Set the vertical scale of the channel'})
        model.append({'element':'variable','name':'vertical_offset','type':float,'unit':'V','read':self.get_vertical_offset,'write':self.set_vertical_offset,'help':'Set the vertical offset of the channel'})
        model.append({'element':'variable','name':'autoscale_factor','type':float,'read':self.get_autoscale_factor,'write':self.set_autoscale_factor,'help':'For setting limits of the vertical scale, units are in number of scope divisions here. WARNING: code security avoid traces too close from screen extrema (at 9 divisions) and result in endless looping. WARNING: the number of vertical divisions might depend on the scope (8 or 10 usually)." '})
        model.append({'element':'variable','name':'active','type':bool,'read':self.is_active,'help':'Returns the current state of the channel.'})
        model.append({'element':'action','name':'autoscale_enable','do':self.set_autoscale_enable,'help':'Enable autoscale mode. This will work best if your scope triggers (i.e. not on AUTO trigger mode with no relevant trigger signal).'})
        model.append({'element':'action','name':'autoscale_disable','do':self.set_autoscale_disable,'help':'Disnable autoscale mode.'})
        return model


   




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Optical frame'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -C VISA -A GPIB0::2::INSTR -c 3,5 -p 30
    load {self.name} driver using VISA and local network address 192.168.0.4 and set the power to 30 mW to slots 3 and 5
    
    autolab driver -D nickname -c 3 -w 1550.1
    Similar to previous one but using the device's nickname as defined in the local_config.ini, and set the wavelength to 1550.1

    Note: Arbitrary waveform available only using a python terminal
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels",type=str, dest="channels", default=None, help="Set the slots to act on/acquire from." )
        parser.add_argument("-p", "--power",type=str, dest="power", default=None, help="Set the power in mW." )
        parser.add_argument("-w", "--wavelength",type=str, dest="wavelength", default=None, help="Set the wavelength in nm." )
        
        return parser

    def do_something(self,args):
        if args.channels:
            for chan in args.channels.split(','):
                assert f'{chan}' in getattr(self.Instance,f'slot_names').keys()
                name_sub_module = getattr(self.Instance,f'slot_names')[f'{chan}']
                sub_module = getattr(self.Instance,name_sub_module)
                if args.power:
                    func_name = 'set_power'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)(args.power)
                if args.wavelength:
                    func_name = 'set_wavelength'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)(args.wavelength)


    def exit(self):
        self.Instance.close()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Oscilloscope'
    
class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Extra infos:  ----------------
- Datas are obtained in a binary format: int8 
- Header is composed as follow:
<format>, <type>, <points>, <count> , <X increment>, <X origin>, < X reference>, <Y increment>, <Y origin>, <Y reference>, <coupling>, <X display range>, <X display origin>, <Y display range>, <Y display origin>, <date>,
<time>, <frame model #>, <acquisition mode>, <completion>, <X units>, <Y units>, <max bandwidth limit>, <min bandwidth limit>    
- To retrieve datas (in "Units")
Y-axis Units = data value * Yincrement + Yorigin (analog channels) 
X-axis Units = data index * Xincrement + Xorigin

----------------  Examples:  ----------------

usage:    autolab driver [options] args
        
    autolab driver -D {self.name} -A 192.168.0.3 -C VXI11 -o my_output_file -c 1
    result in saving two files for the temporal trace of channel 1, the data and the scope parameters, called respectively my_output_file_DSACHAN1 and my_output_file_DSACHAN1.log
    
    autolab driver -D nickname -o my_output_file -c 1,2
    Same as previous one but with 4 output files, two for each channel (1 and 2) and using the device nickname as defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from." )
        parser.add_argument("-o", "--filename", type=str, dest="filename", default=None, help="Set the name of the output file" )
        parser.add_argument("-m", "--measure", type=str, dest="measure", default=None, help="Set measurment number" )
        parser.add_argument("-F", "--force",action="store_true", dest="force", default=None, help="Allows overwriting file" )
        parser.add_argument("-t", "--trigger", dest="trigger",action="store_true", help="Trigger the scope once" )
        parser.add_argument("-f", "--format", type=str, dest="format", default=None, help="Change data encoding" )
        
        return parser

    def do_something(self,args):
        if args.format:
            getattr(self.Instance,'set_type')(args.format)
        if args.trigger:
            getattr(self.Instance,'single')()
        if args.measure:
            for i in range(args.measure):
                getattr(self.Instance,'stop')()
                print(str(i+1))
                getattr(self.Instance,'get_data_channels')(channels=args.channels.split(','))
                getattr(self.Instance,'save_data_channels')(filename=str(i+1),channels=args.channels.split(','),FORCE=args.force)
                getattr(self.Instance,'run')()
                time.sleep(0.050)
        elif args.filename:
            getattr(self.Instance,'get_data_channels')(channels=args.channels.split(','))
            getattr(self.Instance,'save_data_channels')(filename=args.filename,channels=args.channels.split(','),FORCE=args.force)
  

    def exit(self):
        self.Instance.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- signal recovery 7124 
- signal recovery 7280 (VISA only)
"""

import time

class Driver():

    def __init__(self):
        pass 

    def get_id(self):
        return str(self.query('ID'))+' VER '+str(self.query('VER'))

    def wait_four_time_constant(self): # see manual why
        time.sleep(4*self.get_time_constant())

    def get_magnitude(self):
        return float(self.query("MAG.").split('=')[1])

    def get_phase(self):
        return float(self.query("PHA.").split('=')[1])
    
    def get_ref_frequency(self):
        return float(self.query('FRQ.').split('=')[1])

    def get_time_constant(self):
        return float(self.query('TC.').split('=')[1])
    
    def get_sensitivity(self):
        return float(self.query('SEN.').split('=')[1])
    
    
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'time_constant','type':float,'read':self.get_time_constant, 'help':'Filter time constant control.'})        
        model.append({'element':'variable','name':'sensitivity','type':float,'read':self.get_sensitivity, 'help':'Full-scale sensitivity control.'})        
        model.append({'element':'variable','name':'ref_frequency','type':float,'read':self.get_ref_frequency, 'unit':'Hz', 'help':'Reads reference frequency in Hertz.'})        
        model.append({'element':'variable','name':'magnitude','type':float,'read':self.get_magnitude, 'unit':'V', 'help':'Reads magnitude in volts.'})        
        model.append({'element':'variable','name':'phase','type':float,'read':self.get_phase, 'unit':'degrees', 'phase':'Reads Phase in degrees.'})
        model.append({'element':'action','name':'wait_four_time_constant','do':self.wait_four_time_constant,
                       'help':'Wait four time constants. See manual.'})
        return model
    
    
    
    
#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR', **kwargs):
        import visa
        
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        Driver.__init__(self)

    def close(self):
        try : self.controller.close()
        except : pass

    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\n')
        return result
    
    def write(self,command):
        self.controller.write(command)
        
        
class Driver_SOCKET(Driver):
    def __init__(self, address='192.168.0.9', **kwargs):
        import socket
        
        self.BUFFER_SIZE = 40000
        
        self.controller = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.controller.connect((address,int(50000)))    
        Driver.__init__(self)
        
    def write(self,command):
        self.controller.send(command.encode())
        self.controller.recv(self.BUFFER_SIZE)
        

    def query(self,command):
        self.controller.send(command.encode())
        result = self.controller.recv(self.BUFFER_SIZE).decode()
        result = result.strip('\n\x00)\x00')
        return result
    
    def close(self):
        try :
            self.controller.close()
        except :
            pass
        self.controller = None  


############################## Connections classes ##############################
#################################################################################    
    
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Oscilloscope'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Extra infos:  ----------------
    - Datas are obtained in a binary format: int8 
    - To retrieve datas (in "VERTUNIT"), see corresponding log file:
    DATA(VERTUNIT) = DATA(ACQUIRED) * VERTICAL_GAIN - VERTICAL_OFFSET

----------------  Examples:  ----------------

usage:    autolab driver [options] args
        
    autolab driver -D {self.name} -A 192.168.0.3 -C VXI11 -o my_output_file -c 1,2
    Results in saving two files for the trace 1, the data and the scope parameters, called respectively my_output_file_DSACHAN1 and my_output_file_DSACHAN1.log
    
    autolab driver -D nickname -o my_output_file -f WORD 
    Same as previous will save all channels as None are specified and using the device nickname as defined in local_config.ini. the encoding is set to WORD
    
    autolab driver -D nickname -o my_output_file -a 1,3 -b 8
    Vertical autoscale will be enabled for channels 1 and 3. They will both go through the optimization loop 5 times, and aiming 8 vertical divisions.
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from." )
        parser.add_argument("-o", "--filename", type=str, dest="filename", default=None, help="Set the name of the output file" )
        parser.add_argument("-F", "--force",action="store_true", dest="force", default=None, help="Allows overwriting file" )
        parser.add_argument("-t", "--trigger", dest="trigger",action="store_true", help="Trigger the scope once" )
        parser.add_argument("-f", "--format", type=str, dest="format", default=None, help="Change data encoding" )
        parser.add_argument("-a", "--auto_scale", type=str, dest="auto_scale", default=None, help="To allow auto modification of the vertical gain. Argument is a list of all the channels to apply spe mode to. Note if no channel specified. Note: it will work best if your scope triggers (i.e. not on AUTO trigger mode with no relevant trigger signal). WARNING: Specifying more than one channel to apply auto_scale to will result in different trigger events for the acquired channels.")
        parser.add_argument("-b", "--auto_fact", type=float, dest="auto_fact", default=None, help="For setting limits of the vertical gain, units are in number of scope divisions here. WARNING: code security avoid traces too close from screen extrema (at 9 divisions) and result in endless looping. WARNING: the number of vertical divisions might depend on the scope (8 or 10 usually)." )

        return parser

    def do_something(self,args):
        if args.format:
            getattr(self.Instance,'set_encoding')(args.format)
        if args.trigger and not args.filename:
            getattr(self.Instance,'single')()
        if args.auto_scale: # test must be located before args.filename's one
            for chan in args.auto_scale.split(','):
                getattr(getattr(self.Instance,f'channel{chan}'),'set_autoscale_enable')()
        if args.auto_fact: # test must be located before args.filename's one
            for chan in range(1,getattr(self.Instance,'nb_channels')+1):
                getattr(getattr(self.Instance,f'channel{chan}'),'set_autoscale_factor')(args.auto_fact) # all channels instances
        if args.filename:
            acq_channels = args.channels.split(',')
            if args.auto_scale:      # Sort channels for auto ones to be acquired first
                for chan_auto in args.auto_scale.split(','):
                    if chan_auto in acq_channels:
                        acq_channels.remove(chan_auto)
                        acq_channels.insert(0,chan_auto)
            getattr(self.Instance,'get_data_channels')(channels=acq_channels,single=args.trigger)
            getattr(self.Instance,'save_data_channels')(filename=args.filename,channels=acq_channels,FORCE=args.force)
  

    def exit(self):
        self.Instance.close()
        
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Function generator'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}
----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -A TCPIP::192.168.0.4::INSTR -C VISA -a 0.5VP -f 500 -p 1 -o 0.2
    load {self.name} driver using VISA communication protocol with address TCPIP... and set the amplitude to 0.5Volts peak to peak (see help on function -a for other available units), the frequency to 500Hz, the phase to 1 degree and the offset to 0.2V.
    
    autolab driver -D nickname -a 0.5 -f 500
    Similar to previous one but using the device's nickname as defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-p", "--phase", type=str, dest="phase", default=None, help="Set the phase value in degree." )
        parser.add_argument("-o", "--offset", type=str, dest="offset", default=None, help="Set the offset value in Volts." )
        parser.add_argument("-a", "--amplitude", type=str, dest="amplitude", default=None, help="Set the amplitude. Note: The units can be VP(Vpp), VR (Vrms), or DB (dBm)." )
        parser.add_argument("-f", "--frequency", type=str, dest="frequency", default=None, help="Set the frequency in Hz." )
        
        return parser

    def do_something(self,args):
        if args.amplitude:
            getattr(self.Instance,'amplitude')(args.amplitude)
        if args.offset:
            getattr(self.Instance,'offset')(args.offset)
        if args.frequency:
            getattr(self.Instance,'frequency')(args.frequency)
        if args.phase:
            getattr(self.Instance,'phase')(args.phase)
            

    def exit(self):
        self.Instance.close()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import os
import time
from numpy import frombuffer,int8,ndarray


class Driver():
    
    def __init__(self,nb_channels=4):
              
        self.nb_channels = int(nb_channels)
        
        self.write('HORizontal:RECOrdlength?')
        string = self.read()
        length = string[len(':HORIZONTAL:RECORDLENGTH '):]
        self.write('DAT:STAR 1')
        self.write(f'DAT:STOP {length}')
        
        for i in range(1,self.nb_channels+1):
            setattr(self,f'channel{i}',Channel(self,i))
        
    
    ### User utilities
    def get_data_channels(self,channels=[]):
        """Get all channels or the ones specified"""
        self.stop()
        while not self.is_stopped():time.sleep(0.05)
        if channels == []: channels = list(range(1,self.nb_channels+1))
        for i in channels:
            time.sleep(0.1)
            getattr(self,f'channel{i}').get_data_raw()
            getattr(self,f'channel{i}').get_log_data()
        self.run()
        
    def save_data_channels(self,filename,channels=[],FORCE=False):
        if channels == []: channels = list(range(1,self.nb_channels+1))
        for i in channels:
            getattr(self,f'channel{i}').save_data_raw(filename=filename,FORCE=FORCE)
            getattr(self,f'channel{i}').save_log_data(filename=filename,FORCE=FORCE)
        
    ### Trigger functions
    def run(self):
        self.scope.write('ACQUIRE:STATE ON')
    def stop(self):
        self.scope.write('ACQUIRE:STATE OFF')
    def is_stopped(self):
        return '0' in self.query('ACQUIRE:STATE?')


    def get_driver_model(self):
        model = []
        for i in range(1,self.nb_channels+1):
            model.append({'element':'module','name':f'channel{i}','object':getattr(self,f'channel{i}'), 'help':'Channels'})
        model.append({'element':'variable','name':'is_stopped','read':self.is_stopped, 'type':bool,'help':'Query whether scope is stopped'})
        model.append({'element':'action','name':'stop','do':self.stop,'help':'Set stop mode for trigger'})
        model.append({'element':'action','name':'run','do':self.run,'help':'Set run mode for trigger'})
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_VXI11(Driver):
    def __init__(self, address='192.168.0.8', **kwargs):
        import vxi11 as v
    
        self.inst = v.Instrument(address)
        Driver.__init__(self, **kwargs)

    def query(self, command, nbytes=100000000):
        self.write(command)
        return self.read(nbytes)
    def read_raw(self):
        self.inst.read_raw()
    def read(self,nbytes=100000000):
        self.inst.read(nbytes)
    def write(self,cmd):
        self.inst.write(cmd)
    def close(self):
        self.inst.close()
############################## Connections classes ##############################
#################################################################################


class Channel():
    def __init__(self,dev,channel):
        self.channel = int(channel)
        self.dev  = dev
        self.autoscale = False
    
    
    def get_data_raw(self):
        self.dev.write(f'DAT:SOU CH{self.channel}')
        self.dev.write('DAT:ENC FAS')
        self.dev.write('WFMO:BYT_Nr 1')
        self.dev.write('CURV?')
        self.data_raw = self.dev.read_raw()
        self.data_raw = self.data_raw[7:-1]
        return self.data_raw
    def get_log_data(self):
        self.dev.write(f'DAT:SOU CH{self.channel}')
        self.dev.write('WFMO?')
        self.log_data = self.dev.read()
        return self.log_data     
    def get_data(self):
        return frombuffer(self.get_data_raw(),int8)
        
    def save_data_raw(self,filename,FORCE=False):
        temp_filename = f'{filename}_TDS5104BCH{self.channel}'
        if os.path.exists(os.path.join(os.getcwd(),temp_filename)) and not(FORCE):
            print('\nFile ', temp_filename, ' already exists, change filename or remove old file\n')
            return
        f = open(temp_filename,'wb')# Save data
        f.write(self.data_raw)
        f.close()
    def save_log_data(self,filename,FORCE=False):
        temp_filename = f'{filename}_TDS5104BCH{self.channel}.log'
        if os.path.exists(os.path.join(os.getcwd(),temp_filename)) and not(FORCE):
            print('\nFile ', temp_filename, ' already exists, change filename or remove old file\n')
            return
        f = open(temp_filename,'w')
        f.write(self.log_data)
        f.close()
    
    
    def get_data_numerical(self):
        return array_of_float
    def save_data_numerical(self):
        return array_of_float


    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'trace_raw','type':bytes,'read':self.get_data_raw,'help':'Get the current trace in bytes'})
        model.append({'element':'variable','name':'trace','type':ndarray,'read':self.get_data,'help':'Get the current trace in a numpy array of integers'})
        return model
